@@ -65,8 +65,11 @@ val ast_of_acic:
   (** @param env environment from argument_pattern to cic terms
    * @param pat cic_appl_pattern *)
 val instantiate_appl_pattern:
-  (string * Cic.term) list -> CicNotationPt.cic_appl_pattern ->
-    Cic.term
+  mk_appl:('term list -> 'term) -> 
+  mk_implicit:(bool -> 'term) ->
+  term_of_uri : (UriManager.uri -> 'term) ->
+  (string * 'term) list -> CicNotationPt.cic_appl_pattern ->
+    'term
 
-(* hack. seee cicNotation for explanation *)
-val reset: unit -> unit
+val push: unit -> unit
+val pop: unit -> unit
