@@ -23,10 +23,12 @@
  * http://cs.unibo.it/helm/.
  *)
 
-type dependence = CicInspect.S.t * bool
+type dependences = (CicInspect.S.t * bool) list
 
 type conclusion = (int * int * UriManager.uri * int) option
 
-val classify: Cic.context -> Cic.term -> dependence list * conclusion
+val classify: Cic.context -> Cic.term -> dependences * conclusion
 
-val to_string: dependence list * conclusion -> string
+val adjust: Cic.context -> Cic.annterm list -> ?goal:Cic.term -> dependences -> dependences
+
+val to_string: dependences * conclusion -> string
