@@ -25,10 +25,14 @@
 
 val mk_fresh_name:
    Cic.context -> Cic.name -> Cic.name
+val list_fold_right_cps:
+   ('b -> 'c) -> (('b -> 'c) -> 'a -> 'b -> 'c) -> 'a list -> 'b -> 'c
+val list_fold_left_cps:
+   ('b -> 'c) -> (('b -> 'c) -> 'b -> 'a -> 'c) -> 'b -> 'a list -> 'c
 val list_map_cps:
-  ('a list -> 'b) -> (('a -> 'b) -> 'c -> 'b) -> 'c list -> 'b
+   ('b list -> 'c) -> (('b -> 'c) -> 'a -> 'c) -> 'a list -> 'c
 val identity:
-  'a -> 'a
+   'a -> 'a
 val compose:
    ('a -> 'b) -> ('c -> 'a) -> 'c -> 'b
 val fst3:
@@ -36,7 +40,7 @@ val fst3:
 val refine:
    Cic.context -> Cic.term -> Cic.term
 val get_type:
-   Cic.context -> Cic.term -> Cic.term
+   string -> Cic.context -> Cic.term -> Cic.term
 val is_proof:
    Cic.context -> Cic.term -> bool
 val is_sort:
@@ -49,6 +53,8 @@ val is_atomic:
    Cic.term -> bool
 val get_ind_type:
    UriManager.uri -> int -> int * Cic.inductiveType
+val get_ind_names:
+   UriManager.uri -> int -> string list
 val get_default_eliminator:
   Cic.context -> UriManager.uri -> int -> Cic.term -> Cic.term
 val get_ind_parameters:
