@@ -23,4 +23,8 @@
  * http://cs.unibo.it/helm/.
  *)
 
-val optimize_obj: Cic.obj -> Cic.obj
+val optimize_obj: Cic.obj -> Cic.obj * string
+
+val optimize_term: Cic.context -> Cic.term -> Cic.term * string
+
+val debug: bool ref
