@@ -45,8 +45,11 @@ type where    = (hyp * name) option
 type inferred = Cic.annterm
 type pattern  = Cic.annterm
 type body     = Cic.annterm option
+type types    = Cic.anninductiveType list
+type lpsno    = int
 
 type step = Note of note 
+          | Inductive of types * lpsno * note
           | Statement of flavour * name * what * body * note
           | Qed of note
 	  | Id of note
