@@ -33,6 +33,8 @@ type source = string
 
 type prefix = string
 
+type flavour = Cic.object_flavour option
+
 type item = Heading of (string * int)
           | Line of string
 	  | Comment of string
@@ -41,7 +43,7 @@ type item = Heading of (string * int)
           | Coercion of (local * string)
 	  | Notation of (local * string)
 	  | Section of (local * string * string)
-	  | Inline of (local * inline_kind * source * prefix)
+	  | Inline of (local * inline_kind * source * prefix * flavour)
           | Verbatim of string
 	  | Discard of string
 
