@@ -54,7 +54,6 @@ val id_of_annterm: Cic.annterm -> Cic.id
 val params_of_obj: Cic.obj -> UriManager.uri list
 val attributes_of_obj: Cic.obj -> Cic.attribute list
 val projections_of_record: Cic.obj -> UriManager.uri -> UriManager.uri list
-val arity_of_composed_coercion: Cic.obj -> int
 val is_generated: Cic.obj -> bool
 
 (** mk_rels [howmany] [from] 
@@ -71,3 +70,8 @@ val alpha_equivalence: Cic.term -> Cic.term -> bool
  * detects applications without arguments
  *)
 val is_sober: Cic.context -> Cic.term -> bool
+
+val pp_term:
+   (string -> unit) -> Cic.metasenv -> Cic.context -> Cic.term -> unit
+val pp_obj:
+   (string -> unit) -> Cic.obj -> unit
