@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: disambiguateChoices.ml 7749 2007-10-25 16:23:06Z sacerdot $ *)
+(* $Id: disambiguateChoices.ml 9287 2008-12-03 22:53:01Z sacerdot $ *)
 
 open Printf
 
@@ -44,9 +44,10 @@ let lookup_num_by_dsc dsc =
     List.find (has_description dsc) !num_choices
   with Not_found -> raise (Choice_not_found (lazy ("Num with dsc " ^  dsc)))
 
-let mk_choice (dsc, args, appl_pattern) =
+let mk_choice  ~mk_appl ~mk_implicit ~term_of_uri (dsc, args, appl_pattern)=
   dsc,
-  (fun env _ cic_args ->
+  `Sym_interp
+  (fun cic_args ->
     let env',rest =
       let names =
         List.map (function CicNotationPt.IdentArg (_, name) -> name) args
@@ -62,21 +63,30 @@ let mk_choice (dsc, args, appl_pattern) =
         try
          combine_with_rest names cic_args
         with Invalid_argument _ ->
-         raise (Invalid_choice (None, lazy ("The notation " ^ dsc ^ " expects more arguments")))
+         raise (Invalid_choice (lazy (Stdpp.dummy_loc, 
+           "The notation " ^ dsc ^ " expects more arguments")))
     in
      let combined =
-      TermAcicContent.instantiate_appl_pattern env' appl_pattern
+      TermAcicContent.instantiate_appl_pattern 
+        ~mk_appl ~mk_implicit ~term_of_uri env' appl_pattern
      in
       match rest with
          [] -> combined
-       | _::_ -> Cic.Appl (combined::rest))
+       | _::_ -> mk_appl (combined::rest))
 
-let lookup_symbol_by_dsc symbol dsc =
+let lookup_symbol_by_dsc ~mk_appl ~mk_implicit ~term_of_uri symbol dsc =
   try
-    mk_choice
+    mk_choice ~mk_appl ~mk_implicit ~term_of_uri 
       (List.find
         (fun (dsc', _, _) -> dsc = dsc')
         (TermAcicContent.lookup_interpretations symbol))
   with TermAcicContent.Interpretation_not_found | Not_found ->
     raise (Choice_not_found (lazy (sprintf "Symbol %s, dsc %s" symbol dsc)))
 
+let cic_lookup_symbol_by_dsc = lookup_symbol_by_dsc
+  ~mk_implicit:(function 
+     | true -> Cic.Implicit (Some `Type)
+     | false -> Cic.Implicit None)
+  ~mk_appl:(function (Cic.Appl l)::tl -> Cic.Appl (l@tl) | l -> Cic.Appl l)
+  ~term_of_uri:CicUtil.term_of_uri
+;;
