@@ -23,7 +23,7 @@
  * http://cs.unibo.it/helm/.
  *)
 
-(* $Id: cicReduction.ml 8821 2008-07-15 12:00:24Z tassi $ *)
+(* $Id: cicReduction.ml 9538 2009-02-05 10:48:28Z tassi $ *)
 
 (* TODO unify exceptions *)
 
@@ -906,13 +906,18 @@ prerr_endline ("%%%%%%: " ^ CicPp.ppterm term' ^ " <==> " ^ CicPp.ppterm t1);
            else
              false,ugraph
         | (C.Appl l1, C.Appl l2) ->
+           let b, ugraph = 
+             aux test_equality_only context (List.hd l1) (List.hd l2) ugraph
+           in
+           if not b then false, ugraph
+           else
            (try
              List.fold_right2
                (fun  x y (b,ugraph) -> 
                  if b then
-                   aux test_equality_only context x y ugraph
+                   aux true context x y ugraph
                  else
-                   false,ugraph) l1 l2 (true,ugraph)
+                   false,ugraph) (List.tl l1) (List.tl l2) (true,ugraph)
             with
              Invalid_argument _ -> false,ugraph
            )
@@ -975,7 +980,7 @@ prerr_endline ("%%%%%%: " ^ CicPp.ppterm term' ^ " <==> " ^ CicPp.ppterm t1);
              let b'',ugraph''=aux test_equality_only context 
                  outtype1 outtype2 ugraph in
              if b'' then 
-               let b''',ugraph'''= aux test_equality_only context 
+               let b''',ugraph'''= aux true context 
                    term1 term2 ugraph'' in
                List.fold_right2
                  (fun x y (b,ugraph) -> 
@@ -1060,7 +1065,7 @@ prerr_endline ("PREWHD: " ^ CicPp.ppterm t1 ^ " <===> " ^ CicPp.ppterm t2);
    let t2' = whd ?delta:(Some true) ?subst:(Some subst) context t2 in
     debug t1' [t2'] "POSTWHD";
 *)
-let rec convert_machines ugraph =
+let rec convert_machines test_equality_only ugraph =
  function
     [] -> true,ugraph
   | ((k1,env1,ens1,h1,s1),(k2,env2,ens2,h2,s2))::tl ->
@@ -1085,11 +1090,11 @@ let rec convert_machines ugraph =
        in
         match problems with
            None -> false,ugraph
-         | Some problems -> convert_machines ugraph problems
+         | Some problems -> convert_machines true ugraph problems
       else
        res
 in
- convert_machines ugraph
+ convert_machines test_equality_only ugraph
   [R.reduce ~delta:true ~subst context (0,[],[],t1,[]),
    R.reduce ~delta:true ~subst context (0,[],[],t2,[])]
 (*prerr_endline ("POSTWH: " ^ CicPp.ppterm t1' ^ " <===> " ^ CicPp.ppterm t2');*)
