@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: content2presMatcher.ml 8748 2008-06-23 13:49:50Z tassi $ *)
+(* $Id: content2presMatcher.ml 9330 2008-12-12 11:43:39Z tassi $ *)
 
 open Printf
 
@@ -38,7 +38,12 @@ let get_tag term0 =
     subterms := t :: !subterms ; 
     Ast.Implicit
   in
-  let rec aux t = CicNotationUtil.visit_ast ~special_k map_term t
+  let rec aux t = 
+    CicNotationUtil.visit_ast 
+      ~map_xref_option:(fun _ -> None)
+      ~map_case_indty:(fun _ -> None)
+      ~map_case_outtype:(fun _ _ -> None)
+      ~special_k map_term t
   and special_k = function
     | Ast.AttributedTerm (_, t) -> aux t
     | _ -> assert false
