@@ -87,7 +87,7 @@ val filter_map: ('a -> 'b option) -> 'a list -> 'b list (** filter + map *)
 val list_rev_map_filter: ('a -> 'b option) -> 'a list -> 'b list
 val list_rev_map_filter_fold: ('c -> 'a -> 'c * 'b option) -> 'c -> 'a list -> 'c * 'b list
 val list_concat: ?sep:'a list -> 'a list list -> 'a list (**String.concat-like*)
-val list_findopt: ('a -> 'b option) -> 'a list -> 'b option
+val list_findopt: ('a -> int -> 'b option) -> 'a list -> 'b option
 val flatten_map: ('a -> 'b list) -> 'a list -> 'b list
 val list_last: 'a list -> 'a
 val list_mapi: ('a -> int -> 'b) -> 'a list -> 'b list
@@ -100,15 +100,23 @@ val list_iter_default2: ('a -> 'b -> unit) -> 'a list -> 'b -> 'b list -> unit
    length (otherwise it raises Invalid_argument). It stops when the first two
    lists are empty. The third one can be shorter and is padded with a default value. *)
 val list_forall_default3: ('a -> 'b -> 'c -> bool) -> 'a list -> 'b list -> 'c -> 'c list -> bool
+val list_forall_default3_var: ('a -> 'b -> 'c -> bool) -> 'a list -> 'b list -> 'c -> 'c list -> bool
 
+exception FailureAt of int
 
   (** split_nth n l
    * @returns two list, the first contains at least n elements, the second the
-   * remaining one
+   * remaining ones
    * @raise Failure when List.length l < n *)
 val split_nth: int -> 'a list -> 'a list * 'a list
 
+ (** skip [n] [l] skips the first n elements of l *)
+val list_skip : int -> 'a list -> 'a list
+
 val mk_list: 'a -> int -> 'a list
+
+(* makes the list [start; ...; stop - 1] *)
+val list_seq: int -> int -> int list
 (** {2 Debugging & Profiling} *)
 
 type profiler = { profile : 'a 'b. ('a -> 'b) -> 'a -> 'b }
