@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: grafiteMarshal.ml 8754 2008-06-24 14:47:16Z tassi $ *)
+(* $Id: grafiteMarshal.ml 9486 2009-01-13 10:54:34Z tassi $ *)
 
 type ast_command = (Cic.term,Cic.obj) GrafiteAst.command
 type moo = ast_command list
@@ -44,6 +44,8 @@ let rehash_cmd_uris =
   | GrafiteAst.Default (loc, name, uris) ->
       let uris = List.map rehash_uri uris in
       GrafiteAst.Default (loc, name, uris)
+  | GrafiteAst.PreferCoercion (loc, uri) ->
+      GrafiteAst.PreferCoercion (loc, CicUtil.rehash_term uri)
   | GrafiteAst.Coercion (loc, uri, close, arity, saturations) ->
       GrafiteAst.Coercion (loc, CicUtil.rehash_term uri, close, arity, saturations)
   | GrafiteAst.Index (loc, key, uri) ->
