@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: grafiteTypes.ml 8052 2008-01-10 22:32:03Z tassi $ *)
+(* $Id: grafiteTypes.ml 9486 2009-01-13 10:54:34Z tassi $ *)
 
 exception Option_error of string * string
 exception Statement_error of string
@@ -44,21 +44,11 @@ type proof_status =
       (* Status in which the proof could be while it is being processed by the
       * engine. No status entering/exiting the engine could be in it. *)
 
-(* REMOVE
-module StringMap = Map.Make (String)
-type option_value =
-  | String of string
-  | Int of int
-*)
-(* type options = option_value StringMap.t *)
-(* let no_options = StringMap.empty *)
-
 type status = {
   moo_content_rev: GrafiteMarshal.moo;
   proof_status: proof_status;
-(*   options: options; *)
   objects: UriManager.uri list;
-  coercions: UriManager.uri list;
+  coercions: CoercDb.coerc_db;
   universe:Universe.universe;  
   baseuri: string;
 }
