@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: grafiteTypes.mli 8052 2008-01-10 22:32:03Z tassi $ *)
+(* $Id: grafiteTypes.mli 9486 2009-01-13 10:54:34Z tassi $ *)
 
 exception Option_error of string * string
 exception Statement_error of string
@@ -42,20 +42,11 @@ type proof_status =
   | Proof of ProofEngineTypes.proof
   | Intermediate of Cic.metasenv
 
-(*
-type option_value =
-  | String of string
-  | Int of int
-type options
-val no_options: options
-*)
-
 type status = {
   moo_content_rev: GrafiteMarshal.moo;
   proof_status: proof_status;                             (** logical status *)
-(*   options: options; *)
   objects: UriManager.uri list;  (** in-scope objects *)
-  coercions: UriManager.uri list; (** defined coercions *)
+  coercions: CoercDb.coerc_db;
   universe:Universe.universe;  (** universe of terms used by auto *)
   baseuri: string;
 }
