@@ -29,7 +29,9 @@
 val parse_environment:
  include_paths:string list ->
  string ->
-  DisambiguateTypes.environment * DisambiguateTypes.multiple_environment
+  LexiconAst.alias_spec DisambiguateTypes.Environment.t *
+  LexiconAst.alias_spec list DisambiguateTypes.Environment.t
+
 
 (** @param fname file from which load notation *)
 val load_notation: include_paths:string list -> string -> LexiconEngine.status
