@@ -44,7 +44,10 @@ type statement =
 
 val parse_statement: Ulexing.lexbuf -> statement  (** @raise End_of_file *)
 
-val statement: statement Grammar.Entry.e
+val statement: unit -> statement Grammar.Entry.e
 
 (* this callback is called on every include command *)
 val set_callback: (string -> unit) -> unit 
+
+val push : unit -> unit
+val pop : unit -> unit
