@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: print_grammar.ml 8748 2008-06-23 13:49:50Z tassi $ *)
+(* $Id: print_grammar.ml 9021 2008-09-26 16:38:29Z tassi $ *)
 
 open Gramext 
 
@@ -263,7 +263,7 @@ let rec visit_entries fmt todo pped =
 ;;
 
 let ebnf_of_term () =
-  let g_entry = Grammar.Entry.obj CicNotationParser.term in
+  let g_entry = Grammar.Entry.obj (CicNotationParser.term ()) in
   let buff = Buffer.create 100 in
   let fmt = Format.formatter_of_buffer buff in
   visit_entries fmt [g_entry] [];
