@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: lexiconAstPp.ml 7230 2007-02-12 22:53:46Z fguidi $ *)
+(* $Id: lexiconAstPp.ml 9287 2008-12-03 22:53:01Z sacerdot $ *)
 
 open Printf
 
@@ -33,14 +33,14 @@ let pp_l1_pattern = CicNotationPp.pp_term
 let pp_l2_pattern = CicNotationPp.pp_term
 
 let pp_alias = function
-  | Ident_alias (id, uri) -> sprintf "alias id \"%s\" = \"%s\"" id uri
+  | Ident_alias (id, uri) -> sprintf "alias id \"%s\" = \"%s\"." id uri
   | Symbol_alias (symb, instance, desc) ->
-      sprintf "alias symbol \"%s\" %s= \"%s\""
+      sprintf "alias symbol \"%s\" %s= \"%s\"."
         symb
         (if instance=0 then "" else "(instance "^ string_of_int instance ^ ") ")
         desc
   | Number_alias (instance,desc) ->
-      sprintf "alias num (instance %d) = \"%s\"" instance desc
+      sprintf "alias num (instance %d) = \"%s\"." instance desc
   
 let pp_associativity = function
   | Gramext.LeftA -> "left associative"
