@@ -9,14 +9,7 @@
      \ /   This software is distributed as is, NO WARRANTY.     
       V_______________________________________________________________ *)
 
-(* $Id: nCicPp.mli 8530 2008-05-14 13:45:41Z sacerdot $ *)
-
-val set_ppterm: 
-  (context:NCic.context -> 
-   subst:NCic.substitution -> 
-   metasenv:NCic.metasenv ->
-   ?inside_fix:bool ->
-    NCic.term -> string) -> unit
+(* $Id: nCicPp.mli 9054 2008-10-13 15:51:11Z tassi $ *)
 
 val ppterm: 
   context:NCic.context -> 
@@ -26,15 +19,14 @@ val ppterm:
    NCic.term -> string
 
 val ppcontext:
+  ?sep:string ->
   subst:NCic.substitution -> 
   metasenv:NCic.metasenv ->
   NCic.context -> string 
 
-val trivial_pp_term: 
-   context:NCic.context -> 
-   subst:NCic.substitution -> 
-   metasenv:NCic.metasenv ->
-   ?inside_fix:bool ->
-    NCic.term -> string
+val ppmetasenv:
+  subst:NCic.substitution -> NCic.metasenv -> string
+
+val ppsubst: metasenv:NCic.metasenv -> NCic.substitution -> string
 
 val ppobj: NCic.obj -> string
