@@ -9,17 +9,19 @@
      \ /   This software is distributed as is, NO WARRANTY.     
       V_______________________________________________________________ *)
 
-(* $Id: nCicReduction.mli 8698 2008-06-09 16:02:24Z ricciott $ *)
+(* $Id: nCicReduction.mli 9296 2008-12-05 18:16:01Z tassi $ *)
 
 val whd : 
-  ?delta:int -> ?subst:NCic.substitution -> 
+  ?delta:int -> subst:NCic.substitution -> 
   NCic.context -> NCic.term -> 
     NCic.term
 
+val set_get_relevance : 
+  (metasenv:NCic.metasenv -> subst:NCic.substitution ->
+   NCic.context -> NCic.term -> NCic.term list -> bool list) -> unit
+
 val are_convertible :
-  ?subst:NCic.substitution ->
-  (subst:NCic.substitution ->
-   NCic.context -> NCic.term -> NCic.term list -> bool list) ->
+  metasenv:NCic.metasenv -> subst:NCic.substitution ->
   NCic.context -> NCic.term -> NCic.term -> bool
 
 
@@ -27,3 +29,15 @@ val are_convertible :
    delta reduction; if provided, ~upto is the maximum number of beta redexes
    reduced *)
 val head_beta_reduce: ?delta:int -> ?upto:int -> NCic.term -> NCic.term
+
+type stack_item
+type environment_item
+
+type machine = int * environment_item list * NCic.term * stack_item list
+
+val reduce_machine : 
+     delta:int -> ?subst:NCic.substitution -> NCic.context -> machine -> 
+      machine * bool
+val from_stack : stack_item -> machine
+val unwind : machine -> NCic.term
+
