@@ -9,7 +9,7 @@
      \ /   This software is distributed as is, NO WARRANTY.     
       V_______________________________________________________________ *)
 
-(* $Id: nCicTypeChecker.ml 8867 2008-07-23 16:29:35Z tassi $ *)
+(* $Id: nCicTypeChecker.ml 9332 2008-12-12 12:08:16Z tassi $ *)
 
 module C = NCic 
 module Ref = NReference
@@ -121,25 +121,29 @@ let debruijn uri number_of_types context =
   aux (List.length context)
 ;;
 
-let sort_of_prod ~metasenv ~subst context (name,s) (t1, t2) =
+let sort_of_prod ~metasenv ~subst context (name,s) t (t1, t2) =
    let t1 = R.whd ~subst context t1 in
    let t2 = R.whd ~subst ((name,C.Decl s)::context) t2 in
    match t1, t2 with
    | C.Sort _, C.Sort C.Prop -> t2
-   | C.Sort (C.Type u1), C.Sort (C.Type u2) -> C.Sort (C.Type (u1@u2)) 
+   | C.Sort (C.Type u1), C.Sort (C.Type u2) ->
+      C.Sort (C.Type (NCicEnvironment.max u1 u2))
    | C.Sort C.Prop,C.Sort (C.Type _) -> t2
-   | C.Meta (_,(_,(C.Irl 0 | C.Ctx []))), C.Sort _
-   | C.Meta (_,(_,(C.Irl 0 | C.Ctx []))), C.Meta (_,(_,(C.Irl 0 | C.Ctx [])))
-   | C.Sort _, C.Meta  (_,(_,(C.Irl 0 | C.Ctx []))) -> t2
-   | _ -> 
+   | C.Meta (_,(_,(C.Irl 0 | C.Ctx []))), C.Sort _ -> t2
+   | C.Meta (_,(_,(C.Irl 0 | C.Ctx []))), C.Meta (i,(_,(C.Irl 0 | C.Ctx [])))
+   | C.Sort _, C.Meta  (i,(_,(C.Irl 0 | C.Ctx []))) ->
+        NCic.Meta (i,(0, C.Irl 0))
+   | x, (C.Sort _ | C.Meta (_,(_,(C.Irl 0 | C.Ctx []))))
+   | _, x -> 
+      let y, context = 
+        if x == t1 then s, context else t, ((name,C.Decl s)::context)
+      in
       raise (TypeCheckerFailure (lazy (Printf.sprintf
-        "Prod: expected two sorts, found = %s, %s" 
-         (PP.ppterm ~subst ~metasenv ~context t1) 
-         (PP.ppterm ~subst ~metasenv ~context t2))))
+        "%s is expected to be a type, but its type is %s that is not a sort" 
+         (PP.ppterm ~subst ~metasenv ~context y) 
+         (PP.ppterm ~subst ~metasenv ~context x))))
 ;;
 
-(* REMINDER: eat_prods was here *)
-
 (* instantiate_parameters ps (x1:T1)...(xn:Tn)C                             *)
 (* returns ((x_|ps|:T_|ps|)...(xn:Tn)C){ps_1 / x1 ; ... ; ps_|ps| / x_|ps|} *)
 let rec instantiate_parameters params c =
@@ -226,16 +230,32 @@ let rec eat_or_subst_lambdas
   | (_, te, _, _) -> te, k
 ;;
 
+let check_homogeneous_call ~subst context indparamsno n uri reduct tl =
+ let last =
+  List.fold_left
+   (fun k x ->
+     if k = 0 then 0
+     else
+      match R.whd ~subst context x with
+      | C.Rel m when m = n - (indparamsno - k) -> k - 1
+      | _ -> raise (TypeCheckerFailure (lazy 
+         ("Argument "^string_of_int (indparamsno - k + 1) ^ " (of " ^
+         string_of_int indparamsno ^ " fixed) is not homogeneous in "^
+         "appl:\n"^ PP.ppterm ~context ~subst ~metasenv:[] reduct))))
+   indparamsno tl
+ in
+  if last <> 0 then
+   raise (TypeCheckerFailure
+    (lazy ("Non-positive occurence in mutual inductive definition(s) [2]"^
+     NUri.string_of_uri uri)))
+;;
 
-(*CSC l'indice x dei tipi induttivi e' t.c. n < x <= nn *)
-(*CSC questa funzione e' simile alla are_all_occurrences_positive, ma fa *)
-(*CSC dei controlli leggermente diversi. Viene invocata solamente dalla  *)
-(*CSC strictly_positive                                                  *)
-(*CSC definizione (giusta???) tratta dalla mail di Hugo ;-)              *)
-let rec weakly_positive ~subst context n nn uri te =
-(*CSC: Che schifo! Bisogna capire meglio e trovare una soluzione ragionevole!*)
+(* Inductive types being checked for positivity have *)
+(* indexes x s.t. n < x <= nn.                       *)
+let rec weakly_positive ~subst context n nn uri indparamsno posuri te =
+  (*CSC: Not very nice. *)
   let dummy = C.Sort C.Prop in
-  (*CSC: mettere in cicSubstitution *)
+  (*CSC: to be moved in cicSubstitution? *)
   let rec subst_inductive_type_with_dummy _ = function
     | C.Const (Ref.Ref (uri',Ref.Ind (true,0,_))) when NUri.eq uri' uri -> dummy
     | C.Appl ((C.Const (Ref.Ref (uri',Ref.Ind (true,0,lno))))::tl) 
@@ -249,34 +269,34 @@ let rec weakly_positive ~subst context n nn uri te =
      are skipped because we already know that are_all_occurrences_positive
      of uri in te. *)
   let rec aux context n nn te =
-    match R.whd context te with
+    match R.whd ~subst context te with
      | t when t = dummy -> true
      | C.Appl (te::rargs) when te = dummy ->
         List.for_all (does_not_occur ~subst context n nn) rargs
      | C.Prod (name,source,dest) when
         does_not_occur ~subst ((name,C.Decl source)::context) 0 1 dest ->
          (* dummy abstraction, so we behave as in the anonimous case *)
-         strictly_positive ~subst context n nn source &&
-         weakly_positive ~subst ((name,C.Decl source)::context)
-          (n + 1) (nn + 1) uri dest
+         strictly_positive ~subst context n nn indparamsno posuri source &&
+         aux ((name,C.Decl source)::context) (n + 1) (nn + 1) dest
      | C.Prod (name,source,dest) ->
          does_not_occur ~subst context n nn source &&
-         weakly_positive ~subst ((name,C.Decl source)::context)
-          (n + 1) (nn + 1) uri dest
+         aux ((name,C.Decl source)::context) (n + 1) (nn + 1) dest
      | _ ->
        raise (TypeCheckerFailure (lazy "Malformed inductive constructor type"))
    in
      aux context n nn (subst_inductive_type_with_dummy () te)
 
-and strictly_positive ~subst context n nn te =
-  match R.whd context te with
+and strictly_positive ~subst context n nn indparamsno posuri te =
+  match R.whd ~subst context te with
    | t when does_not_occur ~subst context n nn t -> true
-   | C.Rel _ -> true
+   | C.Rel _ when indparamsno = 0 -> true
+   | C.Appl ((C.Rel m)::tl) as reduct when m > n && m <= nn ->
+      check_homogeneous_call ~subst context indparamsno n posuri reduct tl;
+      List.for_all (does_not_occur ~subst context n nn) tl
    | C.Prod (name,so,ta) ->
       does_not_occur ~subst context n nn so &&
-       strictly_positive ~subst ((name,C.Decl so)::context) (n+1) (nn+1) ta
-   | C.Appl ((C.Rel m)::tl) when m > n && m <= nn ->
-      List.for_all (does_not_occur ~subst context n nn) tl
+       strictly_positive ~subst ((name,C.Decl so)::context) (n+1) (nn+1)
+        indparamsno posuri ta
    | C.Appl (C.Const (Ref.Ref (uri,Ref.Ind _) as r)::tl) -> 
       let _,paramsno,tyl,_,i = E.get_checked_indtys r in
       let _,name,ity,cl = List.nth tyl i in
@@ -289,32 +309,16 @@ and strictly_positive ~subst context n nn te =
       ok &&
       List.for_all (does_not_occur ~subst context n nn) arguments &&
       List.for_all 
-       (weakly_positive ~subst ((name,C.Decl ity)::context) (n+1) (nn+1) uri) cl
+       (weakly_positive ~subst ((name,C.Decl ity)::context) (n+1) (nn+1)
+         uri indparamsno posuri) cl
    | _ -> false
        
 (* the inductive type indexes are s.t. n < x <= nn *)
 and are_all_occurrences_positive ~subst context uri indparamsno i n nn te =
-  match R.whd context te with
+  match R.whd ~subst context te with
   |  C.Appl ((C.Rel m)::tl) as reduct when m = i ->
-      let last =
-       List.fold_left
-        (fun k x ->
-          if k = 0 then 0
-          else
-           match R.whd context x with
-           | C.Rel m when m = n - (indparamsno - k) -> k - 1
-           | _ -> raise (TypeCheckerFailure (lazy 
-              ("Argument "^string_of_int (indparamsno - k + 1) ^ " (of " ^
-              string_of_int indparamsno ^ " fixed) is not homogeneous in "^
-              "appl:\n"^ PP.ppterm ~context ~subst ~metasenv:[] reduct))))
-        indparamsno tl
-      in
-       if last = 0 then
-        List.for_all (does_not_occur ~subst context n nn) tl
-       else
-        raise (TypeCheckerFailure
-         (lazy ("Non-positive occurence in mutual inductive definition(s) [2]"^
-          NUri.string_of_uri uri)))
+      check_homogeneous_call ~subst context indparamsno n uri reduct tl;
+      List.for_all (does_not_occur ~subst context n nn) tl
   | C.Rel m when m = i ->
       if indparamsno = 0 then
        true
@@ -322,9 +326,9 @@ and are_all_occurrences_positive ~subst context uri indparamsno i n nn te =
         raise (TypeCheckerFailure
          (lazy ("Non-positive occurence in mutual inductive definition(s) [3]"^
           NUri.string_of_uri uri)))
-  | C.Prod (name,source,dest) when
+   | C.Prod (name,source,dest) when
       does_not_occur ~subst ((name,C.Decl source)::context) 0 1 dest ->
-      strictly_positive ~subst context n nn source &&
+      strictly_positive ~subst context n nn indparamsno uri source &&
        are_all_occurrences_positive ~subst 
         ((name,C.Decl source)::context) uri indparamsno
         (i+1) (n + 1) (nn + 1) dest
@@ -335,7 +339,6 @@ and are_all_occurrences_positive ~subst context uri indparamsno i n nn te =
        are_all_occurrences_positive ~subst ((name,C.Decl source)::context)
         uri indparamsno (i+1) (n + 1) (nn + 1) dest
    | _ ->
-prerr_endline ("MM: " ^ NCicPp.ppterm ~subst ~metasenv:[] ~context te);
      raise
       (TypeCheckerFailure (lazy ("Malformed inductive constructor type " ^
         (NUri.string_of_uri uri))))
@@ -343,6 +346,26 @@ prerr_endline ("MM: " ^ NCicPp.ppterm ~subst ~metasenv:[] ~context te);
 
 exception NotGuarded of string Lazy.t;;
 
+let type_of_branch ~subst context leftno outty cons tycons = 
+ let rec aux liftno context cons tycons =
+   match R.whd ~subst context tycons with
+   | C.Const (Ref.Ref (_,Ref.Ind _)) -> C.Appl [S.lift liftno outty ; cons]
+   | C.Appl (C.Const (Ref.Ref (_,Ref.Ind _))::tl) ->
+       let _,arguments = HExtlib.split_nth leftno tl in
+       C.Appl (S.lift liftno outty::arguments@[cons])
+   | C.Prod (name,so,de) ->
+       let cons =
+        match S.lift 1 cons with
+        | C.Appl l -> C.Appl (l@[C.Rel 1])
+        | t -> C.Appl [t ; C.Rel 1]
+       in
+        C.Prod (name,so, aux (liftno+1) ((name,(C.Decl so))::context) cons de)
+   | _ -> raise (AssertFailure (lazy "type_of_branch"))
+ in
+  aux 0 context cons tycons
+;;
+
+
 let rec typeof ~subst ~metasenv context term =
   let rec typeof_aux context = 
     fun t -> (*prerr_endline (PP.ppterm ~metasenv ~subst ~context t);*)
@@ -364,10 +387,13 @@ let rec typeof ~subst ~metasenv context term =
         try
          let _,c,_,ty = U.lookup_subst n subst in c,ty
         with U.Subst_not_found _ -> try
-         let _,c,ty = U.lookup_meta n metasenv in c,ty
+         let _,c,ty = U.lookup_meta n metasenv in c, ty
+(*          match ty with C.Implicit _ -> assert false | _ -> c,ty *)
         with U.Meta_not_found _ ->
          raise (AssertFailure (lazy (Printf.sprintf
-          "%s not found" (PP.ppterm ~subst ~metasenv ~context t))))
+          "%s not found in:\n%s" (PP.ppterm ~subst ~metasenv ~context t)
+           (PP.ppmetasenv ~subst metasenv)
+          )))
        in
         check_metasenv_consistency t ~subst ~metasenv context canonical_ctx l;
         S.subst_meta l ty
@@ -375,7 +401,7 @@ let rec typeof ~subst ~metasenv context term =
     | C.Prod (name,s,t) ->
        let sort1 = typeof_aux context s in
        let sort2 = typeof_aux ((name,(C.Decl s))::context) t in
-       sort_of_prod ~metasenv ~subst context (name,s) (sort1,sort2)
+       sort_of_prod ~metasenv ~subst context (name,s) t (sort1,sort2)
     | C.Lambda (n,s,t) ->
        let sort = typeof_aux context s in
        (match R.whd ~subst context sort with
@@ -392,7 +418,7 @@ let rec typeof ~subst ~metasenv context term =
     | C.LetIn (n,ty,t,bo) ->
        let ty_t = typeof_aux context t in
        let _ = typeof_aux context ty in
-       if not (R.are_convertible ~subst get_relevance context ty_t ty) then
+       if not (R.are_convertible ~metasenv ~subst context ty_t ty) then
          raise 
           (TypeCheckerFailure 
             (lazy (Printf.sprintf
@@ -461,9 +487,9 @@ let rec typeof ~subst ~metasenv context term =
               let ty_p = typeof_aux context p in
               let ty_cons = typeof_aux context cons in
               let ty_branch = 
-                type_of_branch ~subst context leftno outtype cons ty_cons 0 
+                type_of_branch ~subst context leftno outtype cons ty_cons
               in
-              j+1, R.are_convertible ~subst get_relevance context ty_p ty_branch,
+              j+1, R.are_convertible ~metasenv ~subst context ty_p ty_branch,
               ty_p, ty_branch
             else
               j,false,old_p_ty,old_exp_p_ty
@@ -483,23 +509,6 @@ let rec typeof ~subst ~metasenv context term =
       R.head_beta_reduce (C.Appl res)
     | C.Match _ -> assert false
 
-  and type_of_branch ~subst context leftno outty cons tycons liftno = 
-    match R.whd ~subst context tycons with
-    | C.Const (Ref.Ref (_,Ref.Ind _)) -> C.Appl [S.lift liftno outty ; cons]
-    | C.Appl (C.Const (Ref.Ref (_,Ref.Ind _))::tl) ->
-        let _,arguments = HExtlib.split_nth leftno tl in
-        C.Appl (S.lift liftno outty::arguments@[cons])
-    | C.Prod (name,so,de) ->
-        let cons =
-         match S.lift 1 cons with
-         | C.Appl l -> C.Appl (l@[C.Rel 1])
-         | t -> C.Appl [t ; C.Rel 1]
-        in
-         C.Prod (name,so,
-           type_of_branch ~subst ((name,(C.Decl so))::context) 
-            leftno outty cons de (liftno+1))
-    | _ -> raise (AssertFailure (lazy "type_of_branch"))
-
   (* check_metasenv_consistency checks that the "canonical" context of a
      metavariable is consitent - up to relocation via the relocation list l -
      with the actual context *)
@@ -514,7 +523,7 @@ let rec typeof ~subst ~metasenv context term =
          | 0,_,_::_
          | _,_,[] ->
             raise (AssertFailure (lazy (Printf.sprintf
-             "Local and canonical context %s have different lengths"
+             "(2) Local and canonical context %s have different lengths"
              (PP.ppterm ~subst ~context ~metasenv term))))
          | m,[],_::_ ->
             raise (TypeCheckerFailure (lazy (Printf.sprintf
@@ -525,7 +534,7 @@ let rec typeof ~subst ~metasenv context term =
                 (_,C.Decl t1), (_,C.Decl t2)
               | (_,C.Def (t1,_)), (_,C.Def (t2,_))
               | (_,C.Def (_,t1)), (_,C.Decl t2) ->
-                 if not (R.are_convertible ~subst get_relevance tl t1 t2) then
+                 if not (R.are_convertible ~metasenv ~subst tl t1 t2) then
                   raise 
                       (TypeCheckerFailure 
                         (lazy (Printf.sprintf 
@@ -576,7 +585,7 @@ let rec typeof ~subst ~metasenv context term =
                     with Failure _ -> t)
               | _ -> t
              in
-             if not (R.are_convertible ~subst get_relevance context optimized_t ct)
+             if not (R.are_convertible ~metasenv ~subst context optimized_t ct)
              then
                raise 
                  (TypeCheckerFailure 
@@ -587,7 +596,7 @@ let rec typeof ~subst ~metasenv context term =
                      (PP.ppterm ~subst ~metasenv ~context t))))
           | t, (_,C.Decl ct) ->
               let type_t = typeof_aux context t in
-              if not (R.are_convertible ~subst get_relevance context type_t ct) then
+              if not (R.are_convertible ~metasenv ~subst context type_t ct) then
                 raise (TypeCheckerFailure 
                  (lazy (Printf.sprintf 
                   ("Not well typed metavariable local context: "^^
@@ -597,77 +606,79 @@ let rec typeof ~subst ~metasenv context term =
                   (PP.ppterm ~subst ~metasenv ~context type_t))))
         ) l lifted_canonical_context 
        with
-        Invalid_argument _ ->
+       | Invalid_argument "List.iter2" ->
           raise (AssertFailure (lazy (Printf.sprintf
-           "Local and canonical context %s have different lengths"
+           "(1) Local and canonical context %s have different lengths"
            (PP.ppterm ~subst ~metasenv ~context term))))
 
-  and check_allowed_sort_elimination ~subst ~metasenv r =
-   let mkapp he arg =
-     match he with
-     | C.Appl l -> C.Appl (l @ [arg])
-     | t -> C.Appl [t;arg] in
-   let rec aux context ind arity1 arity2 =
-    let arity1 = R.whd ~subst context arity1 in
-    let arity2 = R.whd ~subst context arity2 in
-      match arity1,arity2 with
-       | C.Prod (name,so1,de1), C.Prod (_,so2,de2) ->
-          if not (R.are_convertible ~subst get_relevance context so1 so2) then
-           raise (TypeCheckerFailure (lazy (Printf.sprintf
-            "In outtype: expected %s, found %s"
-            (PP.ppterm ~subst ~metasenv ~context so1)
-            (PP.ppterm ~subst ~metasenv ~context so2)
-            )));
-          aux ((name, C.Decl so1)::context)
-           (mkapp (S.lift 1 ind) (C.Rel 1)) de1 de2
-       | C.Sort _, C.Prod (name,so,ta) ->
-          if not (R.are_convertible ~subst get_relevance context so ind) then
-           raise (TypeCheckerFailure (lazy (Printf.sprintf
-            "In outtype: expected %s, found %s"
-            (PP.ppterm ~subst ~metasenv ~context ind)
-            (PP.ppterm ~subst ~metasenv ~context so)
-            )));
-          (match arity1, R.whd ~subst ((name,C.Decl so)::context) ta with
-            | (C.Sort C.Type _, C.Sort _)
-            | (C.Sort C.Prop, C.Sort C.Prop) -> ()
-            | (C.Sort C.Prop, C.Sort C.Type _) ->
-        (* TODO: we should pass all these parameters since we
-         * have them already *)
-                let _,leftno,itl,_,i = E.get_checked_indtys r in
-                let itl_len = List.length itl in
-                let _,_,_,cl = List.nth itl i in
-                let cl_len = List.length cl in
-                 (* is it a singleton or empty non recursive and non informative
-                    definition? *)
-                 if not
-                  (cl_len = 0 ||
-                   (itl_len = 1 && cl_len = 1 &&
-                    is_non_informative leftno
-                     (let _,_,x = List.hd cl in x)))
-                 then
-                  raise (TypeCheckerFailure (lazy
-                   ("Sort elimination not allowed")));
-          | _,_ -> ())
-       | _,_ -> ()
-   in
-    aux 
-
  in 
    typeof_aux context term
 
+and check_allowed_sort_elimination ~subst ~metasenv r =
+  let mkapp he arg =
+    match he with
+    | C.Appl l -> C.Appl (l @ [arg])
+    | t -> C.Appl [t;arg] in
+  let rec aux context ind arity1 arity2 =
+   let arity1 = R.whd ~subst context arity1 in
+   let arity2 = R.whd ~subst context arity2 in
+     match arity1,arity2 with
+      | C.Prod (name,so1,de1), C.Prod (_,so2,de2) ->
+         if not (R.are_convertible ~metasenv ~subst context so1 so2) then
+          raise (TypeCheckerFailure (lazy (Printf.sprintf
+           "In outtype: expected %s, found %s"
+           (PP.ppterm ~subst ~metasenv ~context so1)
+           (PP.ppterm ~subst ~metasenv ~context so2)
+           )));
+         aux ((name, C.Decl so1)::context)
+          (mkapp (S.lift 1 ind) (C.Rel 1)) de1 de2
+      | C.Sort _, C.Prod (name,so,ta) ->
+         if not (R.are_convertible ~metasenv ~subst context so ind) then
+          raise (TypeCheckerFailure (lazy (Printf.sprintf
+           "In outtype: expected %s, found %s"
+           (PP.ppterm ~subst ~metasenv ~context ind)
+           (PP.ppterm ~subst ~metasenv ~context so)
+           )));
+         (match arity1, R.whd ~subst ((name,C.Decl so)::context) ta with
+           | (C.Sort C.Type _, C.Sort _)
+           | (C.Sort C.Prop, C.Sort C.Prop) -> ()
+           | (C.Sort C.Prop, C.Sort C.Type _) ->
+       (* TODO: we should pass all these parameters since we
+        * have them already *)
+               let _,leftno,itl,_,i = E.get_checked_indtys r in
+               let itl_len = List.length itl in
+               let _,itname,ittype,cl = List.nth itl i in
+               let cl_len = List.length cl in
+                (* is it a singleton, non recursive and non informative
+                   definition or an empty one? *)
+                if not
+                 (cl_len = 0 ||
+                  (itl_len = 1 && cl_len = 1 &&
+                   let _,_,constrty = List.hd cl in
+                     is_non_recursive_singleton 
+                       ~subst r itname ittype constrty &&
+                     is_non_informative ~metasenv ~subst leftno constrty))
+                then
+                 raise (TypeCheckerFailure (lazy
+                  ("Sort elimination not allowed")));
+         | _,_ -> ())
+      | _,_ -> ()
+  in
+   aux 
+
 and eat_prods ~subst ~metasenv context he ty_he args_with_ty = 
   let rec aux ty_he = function 
   | [] -> ty_he
   | (arg, ty_arg)::tl ->
       match R.whd ~subst context ty_he with 
       | C.Prod (_,s,t) ->
-          if R.are_convertible ~subst get_relevance context ty_arg s then
+          if R.are_convertible ~metasenv ~subst context ty_arg s then
             aux (S.subst ~avoid_beta_redexes:true arg t) tl
           else
             raise 
               (TypeCheckerFailure 
                 (lazy (Printf.sprintf
-                  ("Appl: wrong application of %s: the parameter %s has type"^^
+                  ("Appl: wrong application of %s: the argument %s has type"^^
                    "\n%s\nbut it should have type \n%s\nContext:\n%s\n")
                   (PP.ppterm ~subst ~metasenv ~context he)
                   (PP.ppterm ~subst ~metasenv ~context arg)
@@ -688,14 +699,28 @@ and eat_prods ~subst ~metasenv context he ty_he args_with_ty =
   in
     aux ty_he args_with_ty
 
-and is_non_informative paramsno c =
+and is_non_recursive_singleton ~subst (Ref.Ref (uri,_)) iname ity cty =
+     let ctx = [iname, C.Decl ity] in
+     let cty = debruijn uri 1 [] cty in
+     let len = List.length ctx in
+     let rec aux ctx n nn t =
+       match R.whd ~subst ctx t with
+       | C.Prod (name, src, tgt) ->
+            does_not_occur ~subst ctx n nn src &&
+             aux ((name, C.Decl src) :: ctx) (n+1) (nn+1) tgt
+       | C.Rel k | C.Appl (C.Rel k :: _) when k = nn -> true
+       | _ -> assert false
+     in
+     aux ctx (len-1) len cty
+
+and is_non_informative ~metasenv ~subst paramsno c =
  let rec aux context c =
-   match R.whd context c with
+   match R.whd ~subst context c with
     | C.Prod (n,so,de) ->
-       let s = typeof ~metasenv:[] ~subst:[] context so in
+       let s = typeof ~metasenv ~subst context so in
        s = C.Sort C.Prop && aux ((n,(C.Decl so))::context) de
     | _ -> true in
- let context',dx = split_prods ~subst:[] [] paramsno c in
+ let context',dx = split_prods ~subst [] paramsno c in
   aux context' dx
 
 and check_mutual_inductive_defs uri ~metasenv ~subst leftno tyl = 
@@ -724,11 +749,12 @@ and check_mutual_inductive_defs uri ~metasenv ~subst leftno tyl =
                 let convertible =
                  match item1,item2 with
                    (n1,C.Decl ty1),(n2,C.Decl ty2) ->
-                     n1 = n2 && R.are_convertible ~subst get_relevance context ty1 ty2
+                     n1 = n2 && 
+                     R.are_convertible ~metasenv ~subst context ty1 ty2
                  | (n1,C.Def (bo1,ty1)),(n2,C.Def (bo2,ty2)) ->
                      n1 = n2
-                     && R.are_convertible ~subst get_relevance context ty1 ty2
-                     && R.are_convertible ~subst get_relevance context bo1 bo2
+                     && R.are_convertible ~metasenv ~subst context ty1 ty2
+                     && R.are_convertible ~metasenv ~subst context bo1 bo2
                  | _,_ -> false
                 in
                  if not convertible then
@@ -738,7 +764,7 @@ and check_mutual_inductive_defs uri ~metasenv ~subst leftno tyl =
                  else
                   item1::context
               ) [] sx_context_ty_rev sx_context_te_rev)
-            with Invalid_argument _ -> assert false);
+            with Invalid_argument "List.fold_left2" -> assert false);
            let con_sort = typeof ~subst ~metasenv context te in
            (match R.whd ~subst context con_sort, R.whd ~subst [] ty_sort with
                (C.Sort (C.Type u1) as s1), (C.Sort (C.Type u2) as s2) ->
@@ -946,8 +972,7 @@ and guarded_by_constructors ~subst ~metasenv context t indURI indlen nn =
          ("Too many args for constructor: " ^ String.concat " "
          (List.map (fun x-> PP.ppterm ~subst ~metasenv ~context x) args))))
       in
-      let left, args = HExtlib.split_nth paramsno tl in
-      List.for_all (does_not_occur ~subst context n nn) left &&
+      let _, args = HExtlib.split_nth paramsno tl in
       analyse_instantiated_type rec_params args
    | C.Appl ((C.Match (_,out,te,pl))::_) 
    | C.Match (_,out,te,pl) as t ->
@@ -956,6 +981,9 @@ and guarded_by_constructors ~subst ~metasenv context t indURI indlen nn =
        does_not_occur ~subst context n nn out &&
        does_not_occur ~subst context n nn te &&
        List.for_all (aux context n nn h) pl
+(* IMPOSSIBLE unsless we allow to pass cofix to other fix/cofix as we do for 
+   higher order fix in g_b_destructors.
+
    | C.Const (Ref.Ref (u,(Ref.Fix _| Ref.CoFix _)) as ref)
    | C.Appl(C.Const (Ref.Ref(u,(Ref.Fix _| Ref.CoFix _)) as ref) :: _) as t ->
       let tl = match t with C.Appl (_::tl) -> tl | _ -> [] in
@@ -967,13 +995,14 @@ and guarded_by_constructors ~subst ~metasenv context t indURI indlen nn =
        (fun (_,_,_,_,bo) ->
           aux (context@tys) n nn h (debruijn u len context bo))
        fl
+*)
    | C.Const _
    | C.Appl _ as t -> does_not_occur ~subst context n nn t
  in
    aux context 0 nn false t
                                                                       
 and recursive_args ~subst ~metasenv context n nn te =
-  match R.whd context te with
+  match R.whd ~subst context te with
   | C.Rel _ | C.Appl _ | C.Const _ -> []
   | C.Prod (name,so,de) ->
      (not (does_not_occur ~subst context n nn so)) ::
@@ -1060,13 +1089,13 @@ and type_of_constant ((Ref.Ref (uri,_)) as ref) =
      ty
   | _ -> raise (AssertFailure (lazy "type_of_constant: environment/reference"))
 
-and get_relevance ~subst context t args = 
-   let ty = typeof ~subst ~metasenv:[] context t in
+and get_relevance ~metasenv ~subst context t args = 
+   let ty = typeof ~subst ~metasenv context t in
    let rec aux context ty = function
      | [] -> [] 
      | arg::tl -> match R.whd ~subst context ty with
        | C.Prod (_,so,de) -> 
-           let sort = typeof ~subst ~metasenv:[] context so in
+           let sort = typeof ~subst ~metasenv context so in
            let new_ty = S.subst ~avoid_beta_redexes:true arg de in
            (*prerr_endline ("so: " ^ PP.ppterm ~subst ~metasenv:[]
 	     ~context so);
@@ -1079,14 +1108,14 @@ and get_relevance ~subst context t args =
      	      | C.Meta _ -> true::(aux context new_ty tl)
               | _ -> raise (TypeCheckerFailure (lazy (Printf.sprintf
      	        "Prod: the type %s of the source of %s is not a sort" 
-     	         (PP.ppterm ~subst ~metasenv:[] ~context sort)
-     		 (PP.ppterm ~subst ~metasenv:[] ~context so)))))
+     	         (PP.ppterm ~subst ~metasenv ~context sort)
+     		 (PP.ppterm ~subst ~metasenv ~context so)))))
        | _ ->
           raise 
             (TypeCheckerFailure
               (lazy (Printf.sprintf
                 "Appl: %s is not a function, it cannot be applied"
-                (PP.ppterm ~subst ~metasenv:[] ~context
+                (PP.ppterm ~subst ~metasenv ~context
                  (let res = List.length tl in
                   let eaten = List.length args - res in
                    (C.Appl
@@ -1105,7 +1134,7 @@ let typecheck_context ~metasenv ~subst context =
        | name,C.Def (te,ty) ->
          ignore (typeof ~metasenv ~subst:[] context ty);
          let ty' = typeof ~metasenv ~subst:[] context te in
-          if not (R.are_convertible ~subst get_relevance context ty' ty) then
+          if not (R.are_convertible ~metasenv ~subst context ty' ty) then
            raise (AssertFailure (lazy (Printf.sprintf (
             "the type of the definiens for %s in the context is not "^^
             "convertible with the declared one.\n"^^
@@ -1143,7 +1172,7 @@ let typecheck_subst ~metasenv subst =
       typecheck_context ~metasenv ~subst context;
       ignore (typeof ~metasenv ~subst context ty);
       let ty' = typeof ~metasenv ~subst context bo in
-       if not (R.are_convertible ~subst get_relevance context ty' ty) then
+       if not (R.are_convertible ~metasenv ~subst context ty' ty) then
         raise (AssertFailure (lazy (Printf.sprintf (
          "the type of the definiens for %d in the substitution is not "^^
          "convertible with the declared one.\n"^^
@@ -1164,7 +1193,7 @@ let typecheck_obj (uri,_height,metasenv,subst,kind) =
    | C.Constant (relevance,_,Some te,ty,_) ->
       let _ = typeof ~subst ~metasenv [] ty in
       let ty_te = typeof ~subst ~metasenv [] te in
-      if not (R.are_convertible ~subst get_relevance [] ty_te ty) then
+      if not (R.are_convertible ~metasenv ~subst [] ty_te ty) then
        raise (TypeCheckerFailure (lazy (Printf.sprintf (
         "the type of the body is not convertible with the declared one.\n"^^
         "inferred type:\n%s\nexpected type:\n%s")
@@ -1196,7 +1225,7 @@ let typecheck_obj (uri,_height,metasenv,subst,kind) =
       in
       List.iter2 (fun (_,_,x,ty,_) bo ->
        let ty_bo = typeof ~subst ~metasenv types bo in
-       if not (R.are_convertible ~subst get_relevance types ty_bo ty)
+       if not (R.are_convertible ~metasenv ~subst types ty_bo ty)
        then raise (TypeCheckerFailure (lazy ("(Co)Fix: ill-typed bodies")))
        else
         if inductive then begin
@@ -1272,4 +1301,33 @@ E.set_typecheck_obj
     typecheck_obj obj)
 ;;
 
+let _ = NCicReduction.set_get_relevance get_relevance;;
+
+
+let indent = ref 0;;
+let debug = true;;
+let logger =
+    let do_indent () = String.make !indent ' ' in  
+    (function 
+      | `Start_type_checking s ->
+          if debug then
+           prerr_endline (do_indent () ^ "Start: " ^ NUri.string_of_uri s); 
+          incr indent
+      | `Type_checking_completed s ->
+          decr indent;
+          if debug then
+           prerr_endline (do_indent () ^ "End: " ^ NUri.string_of_uri s)
+      | `Type_checking_interrupted s ->
+          decr indent;
+          if debug then
+           prerr_endline (do_indent () ^ "Break: " ^ NUri.string_of_uri s)
+      | `Type_checking_failed s ->
+          decr indent;
+          if debug then
+           prerr_endline (do_indent () ^ "Fail: " ^ NUri.string_of_uri s)
+      | `Trust_obj s ->
+          if debug then
+           prerr_endline (do_indent () ^ "Trust: " ^ NUri.string_of_uri s))
+;;
+(* let _ = set_logger logger ;; *)
 (* EOF *)
