@@ -9,7 +9,7 @@
      \ /   This software is distributed as is, NO WARRANTY.     
       V_______________________________________________________________ *)
 
-(* $Id: nCicTypeChecker.mli 8530 2008-05-14 13:45:41Z sacerdot $ *)
+(* $Id: nCicTypeChecker.mli 9543 2009-02-11 16:27:33Z tassi $ *)
 
 (* These are the only exceptions that will be raised *)
 exception TypeCheckerFailure of string Lazy.t
@@ -32,3 +32,26 @@ val typeof:
   subst:NCic.substitution -> metasenv:NCic.metasenv -> 
   NCic.context -> NCic.term -> 
     NCic.term
+
+val get_relevance : 
+  metasenv:NCic.metasenv -> subst:NCic.substitution ->
+  NCic.context -> NCic.term -> NCic.term list -> bool list
+
+(* type_of_branch subst context leftno outtype 
+ *   (constr @ lefts) (ty_constr @ lefts)  *)
+val type_of_branch : 
+  subst:NCic.substitution ->
+    NCic.context -> int -> NCic.term -> NCic.term -> NCic.term -> 
+     NCic.term
+
+(* ind = indtype @ lefts
+ * arity1 = constructor type @ lefts
+ * arity2 = outtype *)
+val check_allowed_sort_elimination : 
+  subst:NCic.substitution ->
+  metasenv:NCic.metasenv ->
+  NReference.reference -> NCic.context -> 
+    NCic.term -> NCic.term -> NCic.term -> unit
+
+val debruijn: NUri.uri -> int -> NCic.context -> NCic.term -> NCic.term
+
