@@ -9,7 +9,7 @@
      \ /   This software is distributed as is, NO WARRANTY.     
       V_______________________________________________________________ *)
 
-(* $Id: nReference.mli 8602 2008-05-19 20:58:36Z tassi $ *)
+(* $Id: nReference.mli 9543 2009-02-11 16:27:33Z tassi $ *)
 
 exception IllFormedReference of string Lazy.t
 
@@ -23,11 +23,15 @@ type spec =
 
 type reference = private Ref of NUri.uri * spec
 
+val reference_of_spec: NUri.uri -> spec -> reference
+
 val eq: reference -> reference -> bool
 val string_of_reference: reference -> string 
 val reference_of_string: string -> reference
 
 (* given the reference of an inductive type, returns the i-th contructor *)
 val mk_constructor: int -> reference -> reference
+(* given the reference of an inductive type constructor, returns the indty *)
+val mk_indty: bool -> reference -> reference
 val mk_fix: int -> int -> reference -> reference
 val mk_cofix: int -> reference -> reference
