@@ -9,10 +9,13 @@
      \ /   This software is distributed as is, NO WARRANTY.     
       V_______________________________________________________________ *)
 
-(* $Id: oCic2NCic.mli 8535 2008-05-14 15:36:26Z sacerdot $ *)
+(* $Id: oCic2NCic.mli 9332 2008-12-12 12:08:16Z tassi $ *)
 
 val nuri_of_ouri: UriManager.uri -> NUri.uri
 
 val reference_of_ouri: UriManager.uri -> NReference.spec -> NReference.reference
 
 val convert_obj: UriManager.uri -> Cic.obj -> NCic.obj list
+val convert_term: UriManager.uri -> Cic.term -> NCic.term * NCic.obj list
+
+val clear: unit -> unit
