@@ -1,4 +1,5 @@
 (* Copyright (C) 2002, HELM Team.
+
  * 
  * This file is part of HELM, an Hypertextual, Electronic
  * Library of Mathematics, developed at the Computer Science
@@ -30,6 +31,7 @@ let debug = false;;
 let debug_print s = 
   if debug then prerr_endline (Lazy.force s);;
 
+
 let is_propositional context sort = 
   match CicReduction.whd context sort with
   | Cic.Sort Cic.Prop 
@@ -127,6 +129,15 @@ let partition_equalities =
 
 let default_auto maxm _ _ cache _ _ _ _ = [],cache,maxm ;; 
 
+(* giusto per provare che succede 
+let is_unit_equation context metasenv oldnewmeta term =
+  let head, metasenv, args, newmeta =
+    TermUtil.saturate_term oldnewmeta metasenv context term 0
+  in
+  let newmetas = 
+    List.filter (fun (i,_,_) -> i >= oldnewmeta) metasenv 
+  in
+    Some (args,metasenv,newmetas,head,newmeta) *)
 
 let is_unit_equation context metasenv oldnewmeta term = 
   let head, metasenv, args, newmeta =
@@ -153,7 +164,8 @@ let is_unit_equation context metasenv oldnewmeta term =
     else None
 ;;
 
-let get_candidates universe cache t =
+let get_candidates skip_trie_filtering universe cache t =
+  let t = if skip_trie_filtering then Cic.Meta(0,[]) else t in
   let candidates= 
     (Universe.get_candidates universe t)@(AutoCache.get_candidates cache t)
   in 
@@ -171,11 +183,17 @@ let only signature context metasenv t =
     in
     let consts = MetadataConstraints.constants_of ty in
     let b = MetadataConstraints.UriManagerSet.subset consts signature in
+(*     if b then (prerr_endline ("keeping " ^ (CicPp.ppterm t)); b)  *)
     if b then b 
     else
       let ty' = unfold context ty in
       let consts' = MetadataConstraints.constants_of ty' in
-      MetadataConstraints.UriManagerSet.subset consts' signature 
+      let b = MetadataConstraints.UriManagerSet.subset consts' signature  in
+(*
+	if not b then prerr_endline ("filtering " ^ (CicPp.ppterm t))
+	else prerr_endline ("keeping " ^ (CicPp.ppterm t)); 
+*)
+      b
   with 
   | CicTypeChecker.TypeCheckerFailure _ -> assert false
   | ProofEngineTypes.Fail _ -> false (* unfold may fail *)
@@ -194,11 +212,13 @@ let retrieve_equations dont_filter signature universe cache context metasenv =
         let eq_uri = UriManager.strip_xpointer eq_uri in
         let fake= Cic.Meta(-1,[]) in
         let fake_eq = Cic.Appl [Cic.MutInd (eq_uri,0, []);fake;fake;fake] in
-        let candidates = get_candidates universe cache fake_eq in
+        let candidates = get_candidates false universe cache fake_eq in
         if dont_filter then candidates
-        else 
-          let candidates = List.filter not_default_eq_term candidates in
-          List.filter (only signature context metasenv) candidates 
+        else let eq_uri = UriManager.uri_of_uriref eq_uri 0 None in
+          (* let candidates = List.filter not_default_eq_term candidates in *)
+          List.filter 
+	    (only (MetadataConstraints.UriManagerSet.add eq_uri signature) 
+	       context metasenv) candidates 
 
 let build_equality bag head args proof newmetas maxmeta = 
   match head with
@@ -242,6 +262,7 @@ let empty_tables =
    Saturation.make_passive [],
    Equality.mk_equality_bag)
 
+
 let init_cache_and_tables 
   ?dbd use_library paramod use_context dont_filter universe (proof, goal) 
 =
@@ -263,7 +284,7 @@ let init_cache_and_tables
     (lazy ("ho trovato nella libreria " ^ (string_of_int (List.length lt))));
   let cache = cache_add_list cache context (ct@lt) in  
   let equations = 
-    retrieve_equations dont_filter signature universe cache context metasenv 
+    retrieve_equations dont_filter (* true *) signature universe cache context metasenv 
   in
   debug_print 
     (lazy ("ho trovato equazioni n. "^(string_of_int (List.length equations))));
@@ -514,6 +535,8 @@ let flags_of_params params ?(for_applyS=false) () =
  let bool = bool params in
  let close_more = bool "close_more" false in
  let use_paramod = bool "use_paramod" true in
+ let skip_trie_filtering = bool "skip_trie_filtering" false in
+ let skip_context = bool "skip_context" false in
  let use_only_paramod =
   if for_applyS then true else bool "paramodulation" false in
  let use_library = bool "library"  
@@ -542,6 +565,8 @@ let flags_of_params params ?(for_applyS=false) () =
     AutoTypes.dont_cache_failures = false;
     AutoTypes.maxgoalsizefactor = gsize;
     AutoTypes.do_types = do_type;
+    AutoTypes.skip_trie_filtering = skip_trie_filtering;
+    AutoTypes.skip_context = skip_context;
   }
 
 let universe_of_params metasenv context universe tl =
@@ -608,7 +633,7 @@ let new_metasenv_and_unify_and_t
  with
  | None, _,_,_ -> 
      raise (ProofEngineTypes.Fail (lazy ("FIXME: propaga le tabelle"))) 
- | Some (_,proof''''',_), active,passive,_  ->
+ | Some (_,proof''''',_), active,passive,_  -> 
      proof''''',
      ProofEngineHelpers.compare_metasenvs ~oldmetasenv
        ~newmetasenv:(let _,m,_subst,_,_, _ = proof''''' in m),  active, passive
@@ -689,24 +714,30 @@ let is_in_prop context subst metasenv ty =
   is_propositional context sort
 ;;
 
-let assert_proof_is_valid proof metasenv context goalty =
+exception NotConvertible;;
+
+let check_proof_is_valid proof metasenv context goalty =
   if debug then
     begin
-      let ty,u = typeof metasenv context proof CicUniv.oblivion_ugraph in
-      let b,_ = CicReduction.are_convertible context ty goalty u in
-        if not b then
-          begin
-            let names = 
-              List.map (function None -> None | Some (x,_) -> Some x) context 
-            in
-              debug_print (lazy ("PROOF:" ^ CicPp.pp proof names));
-              debug_print (lazy ("PROOFTY:" ^ CicPp.pp ty names));
-              debug_print (lazy ("GOAL:" ^ CicPp.pp goalty names));
-              debug_print (lazy ("MENV:" ^ CicMetaSubst.ppmetasenv [] metasenv));
-          end;
-        assert b
+      try
+	let ty,u = typeof metasenv context proof CicUniv.oblivion_ugraph in
+	let b,_ = CicReduction.are_convertible context ty goalty u in
+	if not b then raise NotConvertible else b
+      with _ ->
+        let names = 
+          List.map (function None -> None | Some (x,_) -> Some x) context 
+        in
+          debug_print (lazy ("PROOF:" ^ CicPp.pp proof names));
+          (* debug_print (lazy ("PROOFTY:" ^ CicPp.pp ty names)); *)
+          debug_print (lazy ("GOAL:" ^ CicPp.pp goalty names));
+          debug_print (lazy ("MENV:" ^ CicMetaSubst.ppmetasenv [] metasenv));
+        false
     end
-  else ()
+  else true
+;;
+
+let assert_proof_is_valid proof metasenv context goalty =
+  assert (check_proof_is_valid proof metasenv context goalty)
 ;;
 
 let assert_subst_are_disjoint subst subst' =
@@ -807,7 +838,7 @@ type menv = Cic.metasenv
 type subst = Cic.substitution
 type goal = ProofEngineTypes.goal * int * AutoTypes.sort
 let candidate_no = ref 0;;
-type candidate = int * Cic.term
+type candidate = int * Cic.term Lazy.t
 type cache = AutoCache.cache
 type tables = 
   Saturation.active_table * Saturation.passive_table * Equality.equality_bag
@@ -837,8 +868,8 @@ type auto_result =
 (* the status exported to the external observer *)  
 type auto_status = 
   (* context, (goal,candidate) list, and_list, history *)
-  Cic.context * (int * Cic.term * bool * int * (int * Cic.term) list) list * 
-  (int * Cic.term * int) list * Cic.term list
+  Cic.context * (int * Cic.term * bool * int * (int * Cic.term Lazy.t) list) list * 
+  (int * Cic.term * int) list * Cic.term Lazy.t list
 
 let d_prefix l =
   let rec aux acc = function
@@ -859,6 +890,7 @@ let d_goals l =
   in
     aux [] l
 ;;
+
 let calculate_goal_ty (goalno,_,_) s m = 
   try
     let _,cc,goalty = CicUtil.lookup_meta goalno m in
@@ -866,6 +898,7 @@ let calculate_goal_ty (goalno,_,_) s m =
     Some (cc, CicMetaSubst.apply_subst s goalty)
   with CicUtil.Meta_not_found i when i = goalno -> None
 ;;
+
 let calculate_closed_goal_ty (goalno,_,_) s = 
   try
     let cc,_,goalty = List.assoc goalno s in
@@ -873,6 +906,7 @@ let calculate_closed_goal_ty (goalno,_,_) s =
     Some (cc, CicMetaSubst.apply_subst s goalty)
   with Not_found -> None
 ;;
+
 let pp_status ctx status = 
   if debug then 
   let names = Utils.names_of_context ctx in
@@ -890,7 +924,7 @@ let pp_status ctx status =
     | None -> Printf.sprintf "D(%d, _, %d)" gi d
   in
   let string_of_s m su k (ci,ct) gi =
-    Printf.sprintf "S(%d, %s, %s, %d)" gi (pp k) (pp ct) ci
+    Printf.sprintf "S(%d, %s, %s, %d)" gi (pp k) (pp (Lazy.force ct)) ci
   in
   let string_of_ol m su l =
     String.concat " | " 
@@ -1028,6 +1062,15 @@ let list_union l1 l2 =
   (* TODO ottimizzare compare *)
   HExtlib.list_uniq (List.sort compare (l1 @ l1))
 ;;
+let rec eq_todo l1 l2 =
+  match l1,l2 with
+  | (D g1) :: tl1,(D g2) :: tl2 when g1=g2 -> eq_todo tl1 tl2
+  | (S (g1,k1,(c1,lt1),i1)) :: tl1, (S (g2,k2,(c2,lt2),i2)) :: tl2
+    when i1 = i2 && g1 = g2 && k1 = k2 && c1 = c2 ->
+      if Lazy.force lt1 = Lazy.force lt2 then eq_todo tl1 tl2 else false
+  | [],[] -> true
+  | _ -> false
+;;
 let eat_head todo id fl orlist = 
   let rec aux acc = function
   | [] -> [], acc
@@ -1037,7 +1080,7 @@ let eat_head todo id fl orlist =
         | None -> orlist, acc
         | Some (((gno,_,_),_,_,_), todo11) ->
             (* TODO confronto tra todo da ottimizzare *)
-            if gno = id && todo11 = todo then 
+            if gno = id && eq_todo todo11 todo then 
               aux (list_union fl1 acc) tl
             else 
               aux1 todo11
@@ -1128,8 +1171,9 @@ let put_in_subst subst metasenv  (goalno,_,_) canonical_ctx t ty =
   let metasenv = CicMetaSubst.apply_subst_metasenv subst metasenv in
   subst, metasenv
 ;;
+
 let mk_fake_proof metasenv subst (goalno,_,_) goalty context = 
-  None,metasenv,subst ,Cic.Meta(goalno,mk_irl context),goalty, [] 
+  None,metasenv,subst ,(lazy (Cic.Meta(goalno,mk_irl context))),goalty, [] 
 ;;
 let equational_case 
   tables maxm cache depth fake_proof goalno goalty subst context 
@@ -1145,6 +1189,7 @@ let equational_case
         let goal_steps, saturation_steps, timeout =
           max_int,max_int,flags.timeout 
         in
+
         match
           Saturation.given_clause bag maxm status active passive 
             goal_steps saturation_steps timeout
@@ -1208,7 +1253,7 @@ let try_candidate
     let open_goals = order_new_goals metasenv subst open_goals ppterm in
     let open_goals = List.map (fun (x,sort) -> x,depth-1,sort) open_goals in
     incr candidate_no;
-    Some ((!candidate_no,cand),metasenv,subst,open_goals), tables , maxmeta
+    Some ((!candidate_no,lazy cand),metasenv,subst,open_goals), tables , maxmeta
   with 
     | ProofEngineTypes.Fail s -> None,tables, maxm
     | CicUnification.Uncertain s ->  None,tables, maxm
@@ -1221,9 +1266,9 @@ let sort_new_elems =
 
 let applicative_case 
   tables maxm depth subst fake_proof goalno goalty metasenv context universe
-  cache
+  cache flags
 = 
-  let candidates = get_candidates universe cache goalty in
+  let candidates = get_candidates flags.skip_trie_filtering universe cache goalty in
   let tables, elems, maxm = 
     List.fold_left 
       (fun (tables,elems,maxm) cand ->
@@ -1256,14 +1301,14 @@ let equational_and_applicative_case
       else
         applicative_case 
           tables maxm depth s fake_proof goalno 
-            gty m context universe cache 
+            gty m context universe cache flags
     in
     let maxm = maxm1 in
       elems@more_elems, tables, cache, maxm, flags            
   else
     let elems, tables, cache, maxm =
       applicative_case tables maxm depth s fake_proof goalno 
-        gty m context universe cache 
+        gty m context universe cache flags
     in
       elems, tables, cache, maxm, flags  
 ;;
@@ -1288,7 +1333,7 @@ let prunable_for_size flags s m todo =
     | (D g)::tl -> 
 	(match calculate_goal_ty g s m with
           | None -> aux b tl
-          | Some (canonical_ctx, gty) -> 
+	  | Some (canonical_ctx, gty) -> 
             let gsize, _ = 
               Utils.weight_of_term 
 		~consider_metas:false ~count_metas_occurrences:true gty in
@@ -1359,6 +1404,7 @@ let auto_main tables maxm context flags universe cache elems =
     let elems = filter_prune_hint elems in
     match elems with
     | (m, s, size, don, todo, fl)::orlist when !hint <> None ->
+	debug_print (lazy "skip");
         (match !hint with
         | Some i when condition_for_hint i todo ->
             aux tables maxm flags cache orlist
@@ -1367,13 +1413,16 @@ let auto_main tables maxm context flags universe cache elems =
           aux tables maxm flags cache elems)
     | [] ->
         (* complete failure *)
+        debug_print (lazy "give up");
         Gaveup (tables, cache, maxm)
     | (m, s, _, _, [],_)::orlist ->
         (* complete success *)
+        debug_print (lazy "success");
         Proved (m, s, orlist, tables, cache, maxm)
     | (m, s, size, don, (D (_,_,T))::todo, fl)::orlist 
       when not flags.AutoTypes.do_types ->
         (* skip since not Prop, don't even check if closed by side-effect *)
+        debug_print (lazy "skip existential goal");
         aux tables maxm flags cache ((m, s, size, don, todo, fl)::orlist)
     | (m, s, size, don, (S(g, key, c,minsize) as op)::todo, fl)::orlist ->
         (* partial success, cache g and go on *)
@@ -1403,13 +1452,14 @@ let auto_main tables maxm context flags universe cache elems =
         debug_print (lazy ("FAIL: TIMEOUT"));
         Gaveup (tables, cache, maxm)
     | (m, s, size, don, (D (gno,depth,_ as g))::todo, fl)::orlist as status ->
-        (* attack g *)
+        (* attack g *) 
+        debug_print (lazy "attack goal");
         match calculate_goal_ty g s m with
         | None -> 
             (* closed by side effect *)
             debug_print (lazy ("SUCCESS: SIDE EFFECT: " ^ string_of_int gno));
             aux tables maxm flags cache ((m,s,size,don,todo, fl)::orlist)
-        | Some (canonical_ctx, gty) -> 
+        | Some (canonical_ctx, gty) ->
             let gsize, _ = 
               Utils.weight_of_term ~consider_metas:false ~count_metas_occurrences:true gty 
             in
@@ -1570,11 +1620,11 @@ let applyS_tac ~dbd ~term ~params ~universe =
 let auto_tac ~(dbd:HSql.dbd) ~params:(univ,params) ~universe (proof, goal) =
   let _,metasenv,_subst,_,_, _ = proof in
   let _,context,goalty = CicUtil.lookup_meta goal metasenv in
-  let universe = universe_of_params metasenv context universe univ in
   let flags = flags_of_params params () in
+  let universe = universe_of_params metasenv context universe univ in
   let use_library = flags.use_library in
   let tables,cache,newmeta =
-    init_cache_and_tables ~dbd use_library flags.use_only_paramod true 
+    init_cache_and_tables ~dbd use_library flags.use_only_paramod (not flags.skip_context)
       false universe (proof, goal) in
   let tables,cache,newmeta =
     if flags.close_more then
@@ -1590,8 +1640,8 @@ let auto_tac ~(dbd:HSql.dbd) ~params:(univ,params) ~universe (proof, goal) =
   in
   match auto_main tables newmeta context flags universe cache [elem] with
     | Proved (metasenv,subst,_, tables,cache,_) -> 
-        (*prerr_endline 
-          ("TIME:"^string_of_float(Unix.gettimeofday()-.initial_time));*)
+        debug_print (lazy 
+          ("TIME:"^string_of_float(Unix.gettimeofday()-.initial_time)));
         let proof,metasenv =
         ProofEngineHelpers.subst_meta_and_metasenv_in_proof
           proof goal subst metasenv
@@ -1659,17 +1709,159 @@ let solve_rewrite_tac ~params ~universe () =
   ProofEngineTypes.mk_tactic (solve_rewrite_tac ~universe ~params)
 ;;
 
-(* DEMODULATE *)
+(* Demodulate thorem *)
+let open_type ty bo =
+  let rec open_type_aux context ty k args =
+    match ty with 
+      | Cic.Prod (n,s,t) ->
+	  let n' = 
+	    FreshNamesGenerator.mk_fresh_name [] context n ~typ:s ~subst:[] in
+          let entry = match n' with
+	    | Cic.Name _    -> Some (n',(Cic.Decl s))
+	    | Cic.Anonymous -> None
+	  in
+	    open_type_aux (entry::context) t (k+1) ((Cic.Rel k)::args)
+      | Cic.LetIn (n,s,sty,t) ->
+          let entry = Some (n,(Cic.Def (s,sty)))
+	  in
+	    open_type_aux (entry::context) t (k+1) args
+      | _  -> context, ty, args
+  in
+  let context, ty, args = open_type_aux [] ty 1 [] in
+  match args with
+    | [] -> context, ty, bo
+    | _ -> context, ty, Cic.Appl (bo::args)
+;; 
+
+let rec close_type bo ty context =
+  match context with 
+    | [] -> assert_proof_is_valid bo [] [] ty; (bo,ty)
+    | Some (n,(Cic.Decl s))::tl ->
+	close_type (Cic.Lambda (n,s,bo)) (Cic.Prod (n,s,ty)) tl
+    | Some (n,(Cic.Def (s,sty)))::tl ->
+	close_type (Cic.LetIn (n,s,sty,bo)) (Cic.LetIn (n,s,sty,ty)) tl
+    | _ -> assert false
+;; 
+
+let is_subsumed univ context ty =
+  let candidates = Universe.get_candidates univ ty in
+    List.fold_left 
+      (fun res cand ->
+	 match res with
+	   | Some found -> Some found
+	   | None -> 
+	       try 
+                 let mk_irl = CicMkImplicit.identity_relocation_list_for_metavariable in
+		 let metasenv = [(0,context,ty)] in
+		 let fake_proof = None,metasenv,[] , (lazy (Cic.Meta(0,mk_irl context))),ty,[] in
+		 let subst,((_,metasenv,_,_,_,_), open_goals),maxmeta =
+		   (PrimitiveTactics.apply_with_subst ~subst:[] ~maxmeta:0 ~term:cand) (fake_proof,0)
+		 in
+                 let prop_goals, other = split_goals_in_prop metasenv subst open_goals in
+		   if prop_goals = [] then Some cand else None
+	       with 
+		 | ProofEngineTypes.Fail s -> None
+		 | CicUnification.Uncertain s ->  None
+      ) None candidates
+;;
+
+let demodulate_theorem ~universe uri =
+  let eq_uri = 
+    match LibraryObjects.eq_URI () with
+      | Some (uri) -> uri
+      | None -> raise (ProofEngineTypes.Fail (lazy "equality not declared")) in
+  let obj,_ = CicEnvironment.get_cooked_obj CicUniv.empty_ugraph uri
+  in
+  let context,ty,bo =
+    match obj with 
+      | Cic.Constant(n, _, ty ,_, _) -> open_type ty (Cic.Const(uri,[]))
+      | _ -> raise (ProofEngineTypes.Fail (lazy "not a theorem"))
+  in
+  if CicUtil.is_closed ty then 
+    raise (ProofEngineTypes.Fail (lazy ("closed term: dangerous reduction")));
+  let initgoal = [], [], ty in
+  (* compute the signature *)
+  let signature = 
+    let ty_set = MetadataConstraints.constants_of ty in
+    let hyp_set = MetadataQuery.signature_of_hypothesis context [] in
+    let set = MetadataConstraints.UriManagerSet.union ty_set hyp_set in
+      MetadataQuery.close_with_types set [] context 
+  in
+  (* retrieve equations from the universe universe *)
+  let equations = 
+    retrieve_equations true signature universe AutoCache.cache_empty context []
+  in
+  debug_print 
+    (lazy ("ho trovato equazioni n. "^(string_of_int (List.length equations))));
+  let eqs_and_types =
+    HExtlib.filter_map 
+      (fun t -> 
+         let ty,_ =
+           CicTypeChecker.type_of_aux' [] context t CicUniv.oblivion_ugraph
+         in
+         (* retrieve_equations could also return flexible terms *)
+         if is_an_equality ty then Some(t,ty) 
+         else
+           try
+             let ty' = unfold context ty in
+             if is_an_equality ty' then Some(t,ty') else None
+           with ProofEngineTypes.Fail _ -> None) 
+      equations
+  in
+  let bag = Equality.mk_equality_bag () in
+
+  let units, _, newmeta = 
+    partition_unit_equalities context [] (CicMkImplicit.new_meta [] []) bag eqs_and_types 
+  in
+  let table =
+    List.fold_left 
+      (fun tbl eq -> Indexing.index tbl eq) 
+      Indexing.empty units
+  in 
+  let changed,(newproof,newmetasenv, newty) =
+    Indexing.demod bag
+      ([],context,CicUniv.oblivion_ugraph) table initgoal in
+  if changed then
+    begin
+      let oldproof = Equality.Exact bo in
+      let proofterm,_ = 
+        Equality.build_goal_proof (~contextualize:false) (~forward:true) bag
+          eq_uri newproof oldproof ty [] context newmetasenv
+      in
+      if newmetasenv <> [] then 
+	raise (ProofEngineTypes.Fail (lazy ("metasenv not empty")))
+      else
+	begin
+	  assert_proof_is_valid proofterm newmetasenv context newty;
+	  match is_subsumed universe context newty with
+	    | Some t -> raise 
+		(ProofEngineTypes.Fail (lazy ("subsumed by " ^ CicPp.ppterm t)))
+	    | None -> close_type proofterm newty context 
+	end
+    end
+  else (* if newty = ty then *)
+    raise (ProofEngineTypes.Fail (lazy "no progress"))
+  (*else ProofEngineTypes.apply_tactic 
+    (ReductionTactics.simpl_tac
+      ~pattern:(ProofEngineTypes.conclusion_pattern None)) initialstatus*)
+;;      
+
+
+(* NEW DEMODULATE *)
 let demodulate_tac ~dbd ~universe ~params:(univ, params) (proof,goal)= 
   let curi,metasenv,_subst,pbo,pty, attrs = proof in
   let metano,context,ty = CicUtil.lookup_meta goal metasenv in
   let universe = universe_of_params metasenv context universe univ in
   let irl = CicMkImplicit.identity_relocation_list_for_metavariable context in
   let initgoal = [], metasenv, ty in
-  let eq_uri = eq_of_goal ty in
+  let eq_uri = 
+    match LibraryObjects.eq_URI () with
+      | Some (uri) -> uri
+      | None -> raise (ProofEngineTypes.Fail (lazy "equality not declared")) in
+  (* let eq_uri = eq_of_goal ty in *)
   let (active,passive,bag), cache, maxm =
      init_cache_and_tables 
-       ~dbd false true true false universe (proof,goal) 
+       ~dbd false false true true universe (proof,goal) 
   in
   let equalities = (Saturation.list_of_passive passive) in
   (* we demodulate using both actives passives *)
@@ -1678,15 +1870,16 @@ let demodulate_tac ~dbd ~universe ~params:(univ, params) (proof,goal)=
       (fun tbl eq -> Indexing.index tbl eq) 
       (snd active) equalities
   in
-  let changed,(newproof,newmetasenv, newty) = 
-    Indexing.demodulation_goal bag
+  let changed,(newproof,newmetasenv, newty) =
+    (* Indexing.demodulation_goal bag *)
+      Indexing.demod bag
       (metasenv,context,CicUniv.oblivion_ugraph) table initgoal 
   in
   if changed then
     begin
       let opengoal = Equality.Exact (Cic.Meta(maxm,irl)) in
       let proofterm,_ = 
-        Equality.build_goal_proof bag
+        Equality.build_goal_proof (~contextualize:false) bag
           eq_uri newproof opengoal ty [] context metasenv
       in
         let extended_metasenv = (maxm,context,newty)::metasenv in
@@ -1710,5 +1903,5 @@ let demodulate_tac ~dbd ~params ~universe =
 
 let pp_proofterm = Equality.pp_proofterm;;
 
-let revision = "$Revision: 8654 $";;
+let revision = "$Revision: 9202 $";;
 let size_and_depth context metasenv t = 100, 100
