@@ -44,6 +44,11 @@ val demodulate_tac :
   universe:Universe.universe ->
   ProofEngineTypes.tactic
 
+val demodulate_theorem : 
+  universe:Universe.universe -> 
+  UriManager.uri -> 
+  Cic.term * Cic.term
+
 val solve_rewrite_tac:
   params:auto_params ->
   universe:Universe.universe ->
@@ -52,11 +57,11 @@ val solve_rewrite_tac:
 type auto_status = 
   Cic.context * 
   (* or list: goalno, goaltype, grey, depth, candidates: (goalno, c) *)
-  (int * Cic.term * bool * int * (int * Cic.term) list) list * 
+  (int * Cic.term * bool * int * (int * Cic.term Lazy.t) list) list * 
   (* and list *)
   (int * Cic.term * int) list *
   (* last moves *)
-  Cic.term list
+  Cic.term Lazy.t list
 
 val get_auto_status : unit -> auto_status
 val pause: bool -> unit
