@@ -27,7 +27,9 @@ type just = [ `Term of Cic.term | `Auto of Auto.auto_params ]
 
 let mk_just ~dbd ~universe =
  function
-    `Auto params -> Tactics.auto ~dbd ~params ~universe
+    `Auto (l,params) -> 
+       Tactics.auto ~dbd 
+       ~params:(l,("skip_trie_filtering","1")::("skip_context","1")::params) ~universe
   | `Term t -> Tactics.apply t
 ;;
 
