@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: metadataQuery.ml 8450 2008-04-22 19:18:53Z tassi $ *)
+(* $Id: metadataQuery.ml 9204 2008-11-16 10:08:58Z tassi $ *)
 
 open Printf
 
@@ -171,6 +171,8 @@ let close_with_constructors s metasenv context =
 			     List.fold_left 
 			       (fun (j,s) _ -> 
 				  let curi = UriManager.uri_of_uriref uri i (Some j) in
+(*                                     prerr_endline ("adding " ^
+ *                                     (UriManager.string_of_uri curi)); *)
 				    j+1,Constr.UriManagerSet.add curi s) (1,s) cl in
 			     (i+1,s)) (0,bag) tl)
 	       | _ -> assert false)
@@ -272,7 +274,8 @@ let signature_of metasenv goal =
   let ty_set = Constr.constants_of ty in
   let hyp_set = signature_of_hypothesis context metasenv in
   let set = Constr.UriManagerSet.union ty_set hyp_set in
-    close_with_types set metasenv context
+    close_with_constructors (close_with_types set metasenv context)
+    metasenv context
 
 
 let universe_of_goal ~(dbd:HSql.dbd) apply_only metasenv goal =
