@@ -23,12 +23,12 @@
  * http://cs.unibo.it/helm/.
  *)
 
-(* $Id: equality_indexing.ml 6816 2006-09-21 16:01:04Z tassi $ *)
+(* $Id: equality_indexing.ml 8991 2008-09-19 12:47:23Z tassi $ *)
 
 module type EqualityIndex =
   sig
     module PosEqSet : Set.S with type elt = Utils.pos * Equality.equality
-    type t = Discrimination_tree.DiscriminationTreeIndexing(PosEqSet).t
+    type t = Discrimination_tree.Make(Discrimination_tree.CicIndexable)(PosEqSet).t
     val empty : t
     val retrieve_generalizations : t -> Cic.term -> PosEqSet.t
     val retrieve_unifiables : t -> Cic.term -> PosEqSet.t
@@ -36,6 +36,7 @@ module type EqualityIndex =
     val remove_index : t -> Equality.equality -> t
     val index : t -> Equality.equality -> t
     val in_index : t -> Equality.equality -> bool
+    val iter : t -> (Discrimination_tree.CicIndexable.constant_name Discrimination_tree.path -> PosEqSet.t -> unit) -> unit
   end
 
 module DT = 
@@ -49,7 +50,7 @@ struct
 
     module PosEqSet = Set.Make(OrderedPosEquality);;
     
-    include Discrimination_tree.DiscriminationTreeIndexing(PosEqSet)
+    include Discrimination_tree.Make(Discrimination_tree.CicIndexable)(PosEqSet)
     
 
     (* DISCRIMINATION TREES *)
@@ -94,7 +95,7 @@ module PT =
 
     module PosEqSet = Set.Make(OrderedPosEquality);;
     
-    include Discrimination_tree.DiscriminationTreeIndexing(PosEqSet)
+    include Discrimination_tree.Make(Discrimination_tree.CicIndexable)(PosEqSet)
     
 
     (* DISCRIMINATION TREES *)
