@@ -25,7 +25,7 @@
 
 (* let _profiler = <:profiler<_profiler>>;; *)
 
-(* $Id: indexing.ml 7438 2007-06-13 13:01:32Z tassi $ *)
+(* $Id: indexing.ml 9548 2009-02-15 14:57:16Z tassi $ *)
 
 module Index = Equality_indexing.DT (* discrimination tree based indexing *)
 (*
@@ -109,19 +109,73 @@ let check_disjoint_invariant subst metasenv msg =
 ;;
 
 let check_for_duplicates metas msg =
-  if List.length metas <> 
-  List.length (HExtlib.list_uniq (List.sort Pervasives.compare metas)) then
-    begin 
+  let rec aux = function
+    | [] -> true
+    | (m,_,_)::tl -> not (List.exists (fun (i, _, _) -> i = m) tl) && aux tl in
+  let b = aux metas in
+    if not b then  
+      begin 
       prerr_endline ("DUPLICATI " ^ msg);
       prerr_endline (CicMetaSubst.ppmetasenv [] metas);
       assert false
-    end
+      end
+    else ()
+;;
+
+let check_metasenv msg menv =
+  List.iter
+    (fun (i,ctx,ty) -> 
+       try ignore(CicTypeChecker.type_of_aux' menv ctx ty 
+		  CicUniv.empty_ugraph)
+       with 
+	 | CicUtil.Meta_not_found _ -> 
+	     prerr_endline (msg ^ CicMetaSubst.ppmetasenv [] menv);
+	     assert false
+	 | _ -> ()
+    ) menv
+;;
+
+(* the metasenv returned by res must included in the original one,
+due to matching. If it fails, it is probably because we are not 
+demodulating with a unit equality *)
+
+let not_unit_eq ctx eq =
+  let (_,_,(ty,left,right,o),metas,_) = Equality.open_equality eq in
+  let b = 
+  List.exists 
+    (fun (_,_,ty) ->
+       try 
+	 let s,_ = CicTypeChecker.type_of_aux' metas ctx ty CicUniv.empty_ugraph
+	 in s = Cic.Sort(Cic.Prop)
+       with _ -> 
+	 prerr_endline ("ERROR typing " ^ CicPp.ppterm ty); assert false) metas
+  in b
+(*
+if b then prerr_endline ("not a unit equality: " ^ Equality.string_of_equality eq); b *)
+;;
+
+let check_demod_res res metasenv msg =
+  match res with
+    | Some (_, _, menv, _, _) ->
+	let b =
+	  List.for_all
+            (fun (i,_,_) -> 
+	       (List.exists (fun (j,_,_) -> i=j) metasenv)) menv
+	in
+	  if (not b) then
+	    begin
+	      prerr_endline ("extended context " ^ msg);
+	      prerr_endline (CicMetaSubst.ppmetasenv [] menv);
+	    end;
+	b
+    | None -> false
 ;;
 
 let check_res res msg =
   match res with
-      Some (t, subst, menv, ug, eq_found) ->
+    | Some (t, subst, menv, ug, eq_found) ->
         let eqs = Equality.string_of_equality (snd eq_found) in
+        check_metasenv msg menv;
         check_disjoint_invariant subst menv msg;
         check_for_duplicates menv (msg ^ "\nchecking " ^ eqs);
     | None -> ()
@@ -222,6 +276,11 @@ let rec find_matches bag metasenv context ugraph lift_amount term termty =
     | [] -> None
     | candidate::tl ->
         let pos, equality = candidate in
+        (* if not_unit_eq context equality then
+	  begin
+	    prerr_endline "not a unit";
+	    prerr_endline (Equality.string_of_equality equality)
+	  end; *)
         let (_, proof, (ty, left, right, o), metas,_) = 
           Equality.open_equality equality 
         in
@@ -232,13 +291,18 @@ let rec find_matches bag metasenv context ugraph lift_amount term termty =
             let c="eq = "^(Equality.string_of_equality (snd candidate)) ^ "\n"in
             let t="t = " ^ (CicPp.ppterm term) ^ "\n" in
             let m="metas = " ^ (CicMetaSubst.ppmetasenv [] metas) ^ "\n" in
-(*
+            let ms="metasenv =" ^ (CicMetaSubst.ppmetasenv [] metasenv) ^ "\n" in
+            let eq_uri = 
+	      match LibraryObjects.eq_URI () with
+		| Some (uri) -> uri
+		| None -> raise (ProofEngineTypes.Fail (lazy "equality not declared")) in
             let p="proof = "^
-              (CicPp.ppterm(Equality.build_proof_term proof))^"\n" 
+              (CicPp.ppterm(Equality.build_proof_term bag eq_uri [] 0 proof))^"\n" 
             in
-*)
+
               check_for_duplicates metas "gia nella metas";
-              check_for_duplicates (metasenv@metas) ("not disjoint"^c^t^m(*^p*))
+              check_for_duplicates metasenv "gia nel metasenv";
+              check_for_duplicates (metasenv@metas) ("not disjoint"^c^t^m^ms^p)
           end;
         if check && not (fst (CicReduction.are_convertible
                                 ~metasenv context termty ty ugraph)) then (
@@ -249,6 +313,7 @@ let rec find_matches bag metasenv context ugraph lift_amount term termty =
               Founif.matching 
                 metasenv metas context term (S.lift lift_amount c) ugraph
             in
+	    check_metasenv "founif :" metasenv';
             Some (Cic.Rel(1+lift_amount),subst',metasenv',ugraph',candidate)
           in
           let c, other =
@@ -269,7 +334,7 @@ let rec find_matches bag metasenv context ugraph lift_amount term termty =
               try do_match c 
               with Founif.MatchingFailure -> None
             in
-            if Utils.debug_res then ignore (check_res res "find2");
+	      if Utils.debug_res then ignore (check_res res "find2");
             match res with
             | Some (_, s, _, _, _) ->
                 let c' = apply_subst s c in
@@ -302,18 +367,25 @@ let rec find_all_matches ?(unif_fun=Founif.unification)
   let module S = CicSubstitution in
   let module M = CicMetaSubst in
   let module HL = HelmLibraryObjects in
+  (* prerr_endline ("matching " ^  CicPp.ppterm term); *)
   let cmp = !Utils.compare_terms in
+  let check = match termty with C.Implicit None -> false | _ -> true in
   function
     | [] -> []
     | candidate::tl ->
         let pos, equality = candidate in 
-        let (_,_,(ty,left,right,o),metas,_)=Equality.open_equality equality in
+        let (_,_,(ty,left,right,o),metas,_)= Equality.open_equality equality in
+	if check && not (fst (CicReduction.are_convertible
+                                ~metasenv context termty ty ugraph)) then (
+          find_all_matches metasenv context ugraph lift_amount term termty tl
+        ) else
         let do_match c =
           let subst', metasenv', ugraph' =
             unif_fun metasenv metas context term (S.lift lift_amount c) ugraph
           in
           (C.Rel (1+lift_amount),subst',metasenv',ugraph',candidate)
         in
+        
         let c, other =
           if pos = Utils.Left then left, right
           else right, left
@@ -429,10 +501,12 @@ let unification x y z =
   subsumption_aux true x y z
 ;;
 
+(* the target must be disjoint from the equations in the table *)
 let subsumption_aux_all use_unification env table target = 
   let _, _, (ty, left, right, _), tmetas, _ = Equality.open_equality target in
   let _, context, ugraph = env in
   let metasenv = tmetas in
+  check_for_duplicates metasenv "subsumption_aux_all";
   let predicate, unif_fun = 
     if use_unification then
       Unification, Founif.unification
@@ -441,7 +515,7 @@ let subsumption_aux_all use_unification env table target =
   in
   let leftr =
     match left with
-    | Cic.Meta _ when not use_unification -> []   
+    | Cic.Meta _ (*when not use_unification*) -> []   
     | _ ->
         let leftc = get_candidates predicate table left in
         find_all_matches ~unif_fun
@@ -449,7 +523,7 @@ let subsumption_aux_all use_unification env table target =
   in
   let rightr =
         match right with
-          | Cic.Meta _ when not use_unification -> [] 
+          | Cic.Meta _ (*when not use_unification*) -> [] 
           | _ ->
               let rightc = get_candidates predicate table right in
                 find_all_matches ~unif_fun
@@ -464,7 +538,7 @@ let subsumption_aux_all use_unification env table target =
           let what' = Subst.apply_subst subst what in
           let other' = Subst.apply_subst subst other in
           let subst', menv', ug' =
-            unif_fun metasenv m context what' other' ugraph
+            unif_fun [] menv context what' other' ugraph
           in
           (match Subst.merge_subst_if_possible subst subst' with
           | None -> ok_all what leftorright tl
@@ -482,34 +556,48 @@ let subsumption_all x y z =
 ;;
 
 let unification_all x y z = 
-  subsumption_aux_all true x y z
+  prerr_endline "unification_all"; subsumption_aux_all true x y z
 ;;
+
 let rec demodulation_aux bag ?from ?(typecheck=false) 
   metasenv context ugraph table lift_amount term =
-(*  Printf.eprintf "term = %s\n" (CicPp.ppterm term);*)
   let module C = Cic in
   let module S = CicSubstitution in
   let module M = CicMetaSubst in
   let module HL = HelmLibraryObjects in
+  (* prerr_endline ("demodulating " ^ CicPp.ppterm term); *)
+  check_for_duplicates metasenv "in input a demodulation aux";
   let candidates = 
     get_candidates 
       ~env:(metasenv,context,ugraph) (* Unification *) Matching table term 
+  in let candidates = List.filter (fun _,x -> not (not_unit_eq context x)) candidates 
   in
   let res =
     match term with
       | C.Meta _ -> None
       | term ->
-          let termty, ugraph =
-            if typecheck then
-              CicTypeChecker.type_of_aux' metasenv context term ugraph
-            else
-              C.Implicit None, ugraph
+	  let res = 
+	    try
+              let termty, ugraph =
+		if typecheck then
+		  CicTypeChecker.type_of_aux' metasenv context term ugraph
+		else
+		  C.Implicit None, ugraph
+              in
+		find_matches bag metasenv context ugraph 
+		  lift_amount term termty candidates
+            with _ ->  
+	      prerr_endline "type checking error";
+	      prerr_endline ("menv :\n" ^ CicMetaSubst.ppmetasenv [] metasenv);
+	      prerr_endline ("term: " ^ (CicPp.ppterm term));
+	      assert false;
+              (* None *)
           in
-          let res =
-            find_matches bag metasenv context ugraph lift_amount term termty candidates
-          in
-          if Utils.debug_res then ignore(check_res res "demod1"); 
-            if res <> None then
+	  let res = 
+	    (if Utils.debug_res then
+            ignore(check_res res "demod1");
+	    if check_demod_res res metasenv "demod" then res else None) in
+          if res <> None then
               res
             else
               match term with
@@ -521,7 +609,7 @@ let rec demodulation_aux bag ?from ?(typecheck=false)
                              (res, tl @ [S.lift 1 t])
                            else 
                              let r =
-                               demodulation_aux bag ~from:"1" metasenv context ugraph table
+                               demodulation_aux bag ~from:"1" metasenv context ugraph table ~typecheck
                                  lift_amount t
                              in
                                match r with
@@ -534,6 +622,7 @@ let rec demodulation_aux bag ?from ?(typecheck=false)
                           | Some (_, subst, menv, ug, eq_found) ->
                               Some (C.Appl ll, subst, menv, ug, eq_found)
                       )
+(*
                 | C.Prod (nn, s, t) ->
                     let r1 =
                       demodulation_aux bag ~from:"2"
@@ -556,6 +645,7 @@ let rec demodulation_aux bag ?from ?(typecheck=false)
                                     subst, menv, ug, eq_found)
                       )
                 | C.Lambda (nn, s, t) ->
+                    prerr_endline "siam qui";
                     let r1 =
                       demodulation_aux bag
                         metasenv context ugraph table lift_amount s in (
@@ -576,6 +666,7 @@ let rec demodulation_aux bag ?from ?(typecheck=false)
                               Some (C.Lambda (nn, s', (S.lift 1 t)),
                                     subst, menv, ug, eq_found)
                       )
+*)
                 | t ->
                     None
   in
@@ -587,6 +678,16 @@ exception Foo
 
 (** demodulation, when target is an equality *)
 let rec demodulation_equality bag ?from eq_uri newmeta env table target =
+        (*
+          prerr_endline ("demodulation_eq:\n");
+        Index.iter table (fun l -> 
+          let l = Index.PosEqSet.elements l in
+          let l = 
+            List.map (fun (p,e) -> 
+              Utils.string_of_pos p ^ Equality.string_of_equality e) l in
+          prerr_endline (String.concat "\n" l)
+          );
+          *)
   let module C = Cic in
   let module S = CicSubstitution in
   let module M = CicMetaSubst in
@@ -652,7 +753,7 @@ let rec demodulation_equality bag ?from eq_uri newmeta env table target =
   in
 
   let res = 
-    demodulation_aux bag ~from:"3" metasenv' context ugraph table 0 left 
+    demodulation_aux bag ~from:"from3" metasenv' context ugraph table 0 left 
   in
   if Utils.debug_res then check_res res "demod result";
   let newmeta, newtarget = 
@@ -800,8 +901,8 @@ let rec betaexpand_term
   | C.Meta (i, l) -> res, lifted_term
   | term ->
       let termty, ugraph =
-        C.Implicit None, ugraph
-(*         CicTypeChecker.type_of_aux' metasenv context term ugraph *)
+(*        C.Implicit None, ugraph *)
+         CicTypeChecker.type_of_aux' metasenv context term ugraph 
       in
       let candidates = get_candidates Unification table term in
       let r = 
@@ -909,50 +1010,50 @@ let superposition_right bag
 ;;
 
 (** demodulation, when the target is a theorem *)
-let rec demodulation_theorem bag newmeta env table theorem =
+let rec demodulation_theorem bag env table theorem =
   let module C = Cic in
   let module S = CicSubstitution in
   let module M = CicMetaSubst in
   let module HL = HelmLibraryObjects in
+  let eq_uri =
+    match LibraryObjects.eq_URI() with
+    | Some u -> u
+    | None -> assert false in
   let metasenv, context, ugraph = env in
-  let maxmeta = ref newmeta in
-  let term, termty, metas = theorem in
-  let metasenv' = metas in
-  
+  let proof, theo, metas = theorem in
   let build_newtheorem (t, subst, menv, ug, eq_found) =
     let pos, equality = eq_found in
     let (_, proof', (ty, what, other, _), menv',id) = 
       Equality.open_equality equality in
-    let what, other = if pos = Utils.Left then what, other else other, what in
-    let newterm, newty =
-      let bo = Utils.guarded_simpl context (apply_subst subst (S.subst other t)) in
-(*      let bo' = apply_subst subst t in *)
-(*      let name = C.Name ("x_DemodThm_" ^ (string_of_int !demod_counter)) in*)
-(*
-      let newproofold =
-        Equality.ProofBlock (subst, eq_URI, (name, ty), bo', eq_found,
-                              Equality.BasicProof (Equality.empty_subst,term))
-      in
-      (Equality.build_proof_term_old newproofold, bo)
-*)
-      (* TODO, not ported to the new proofs *) 
-      if true then assert false; term, bo
-    in    
-    !maxmeta, (newterm, newty, menv)
-  in  
-  let res =
-    demodulation_aux bag (* ~typecheck:true *) metasenv' context ugraph table 0 termty
+    let peq = 
+      match proof' with
+      | Equality.Exact p -> p
+      | _ -> assert false in
+    let what, other = 
+      if pos = Utils.Left then what, other else other, what in 
+    let newtheo = apply_subst subst (S.subst other t) in
+    let name = C.Name "x" in
+    let body = apply_subst subst t in 
+    let pred = C.Lambda(name,ty,body) in 
+    let newproof =
+      match pos with
+        | Utils.Left ->
+          Equality.mk_eq_ind eq_uri ty what pred proof other peq
+        | Utils.Right ->
+          Equality.mk_eq_ind eq_uri ty what pred proof other peq
+    in
+    newproof,newtheo
   in
+  let res = demodulation_aux bag metas context ugraph table 0 theo in
   match res with
   | Some t ->
-      let newmeta, newthm = build_newtheorem t in
-      let newt, newty, _ = newthm in
-      if Equality.meta_convertibility termty newty then
-        newmeta, newthm
+      let newproof, newtheo = build_newtheorem t in
+      if Equality.meta_convertibility theo newtheo then
+        newproof, newtheo
       else
-        demodulation_theorem bag newmeta env table newthm
+        demodulation_theorem bag env table (newproof,newtheo,[])
   | None ->
-      newmeta, theorem
+      proof,theo
 ;;
 
 (*****************************************************************************)
@@ -961,13 +1062,52 @@ let rec demodulation_theorem bag newmeta env table theorem =
 (**                DEMODULATION_GOAL & SUPERPOSITION_LEFT                   **)
 (*****************************************************************************)
 
+(* new: demodulation of non_equality terms *)
+let build_newg bag context goal rule expansion =
+  let goalproof,_,_ = goal in
+  let (t,subst,menv,ug,eq_found) = expansion in
+  let pos, equality = eq_found in
+  let (_, proof', (ty, what, other, _), menv',id) = 
+    Equality.open_equality equality in
+  let what, other = if pos = Utils.Left then what, other else other, what in
+  let newterm, newgoalproof =
+    let bo = 
+      Utils.guarded_simpl context 
+        (apply_subst subst (CicSubstitution.subst other t)) 
+    in
+    let name = Cic.Name "x" in     
+    let pred = apply_subst subst (Cic.Lambda (name,ty,t)) in 
+    let newgoalproofstep = (rule,pos,id,subst,pred) in
+    bo, (newgoalproofstep::goalproof)
+  in
+  let newmetasenv = (* Founif.filter subst *) menv in
+  (newgoalproof, newmetasenv, newterm)
+;;
+
+let rec demod bag env table goal =
+  let goalproof,menv,t = goal in
+  let _, context, ugraph = env in
+  let res = demodulation_aux bag menv context ugraph table 0 t (~typecheck:true)in
+  match res with
+    | Some newt ->
+	let newg = 
+          build_newg bag context goal Equality.Demodulation newt 
+        in
+        let _,_,newt = newg in
+        if Equality.meta_convertibility t newt then
+          false, goal
+        else
+          true, snd (demod bag env table newg)
+    | None -> 
+	false, goal
+;;
+
 let open_goal g =
   match g with
   | (proof,menv,Cic.Appl[(Cic.MutInd(uri,0,_)) as eq;ty;l;r]) -> 
       (* assert (LibraryObjects.is_eq_URI uri); *)
       proof,menv,eq,ty,l,r
   | _ -> assert false
-;;
 
 let ty_of_goal (_,_,ty) = ty ;;
 
@@ -981,6 +1121,7 @@ let goal_metaconvertibility_eq g1 g2 =
  * C[x] ---> (eq ty unchanged C[x])
  * [posu] is the side of the [unchanged] term in the original goal
  *)
+
 let fix_expansion goal posu (t, subst, menv, ug, eq_f) = 
   let _,_,eq,ty,l,r = open_goal goal in
   let unchanged = if posu = Utils.Left then l else r in
@@ -1008,10 +1149,9 @@ let build_newgoal bag context goal posu rule expansion =
       Utils.guarded_simpl context 
         (apply_subst subst (CicSubstitution.subst other t)) 
     in
-    let bo' = apply_subst subst t in
-    let ty = apply_subst subst ty in 
     let name = Cic.Name "x" in 
-    let newgoalproofstep = (rule,pos,id,subst,Cic.Lambda (name,ty,bo')) in
+    let pred = apply_subst subst (Cic.Lambda (name,ty,t)) in 
+    let newgoalproofstep = (rule,pos,id,subst,pred) in
     bo, (newgoalproofstep::goalproof)
   in
   let newmetasenv = (* Founif.filter subst *) menv in
@@ -1036,6 +1176,10 @@ let superposition_left bag (metasenv, context, ugraph) table goal maxmeta =
       prerr_endline (string_of_int (List.length expansionsl));
       prerr_endline (string_of_int (List.length expansionsr));
       *)
+(*
+      if expansionsl <> [] then prerr_endline "expansionl";
+      if expansionsr <> [] then prerr_endline "expansionr";
+*)
       List.map (build_newgoal bag context goal Utils.Right Equality.SuperpositionLeft) expansionsl
       @
       List.map (build_newgoal bag context goal Utils.Left Equality.SuperpositionLeft) expansionsr
