@@ -23,9 +23,9 @@
  * http://cs.unibo.it/helm/.
  *)
 
-(* $Id: utils.ml 8206 2008-03-11 22:02:58Z sacerdot $ *)
+(* $Id: utils.ml 9146 2008-11-04 15:13:34Z asperti $ *)
 
-let time = true;;
+let time = false;;
 let debug = false;;
 let debug_metas = false;; 
 let debug_res = false;;
@@ -506,12 +506,21 @@ let rec aux_ordering ?(recursion=true) t1 t2 =
   | C.Const _, _ -> Lt
   | _, C.Const _ -> Gt
 
-  | C.MutInd (u1, _, _), C.MutInd (u2, _, _) -> compare_uris u1 u2
+  | C.MutInd (u1, tno1, _), C.MutInd (u2, tno2, _) -> 
+       let res =  compare_uris u1 u2 in
+       if res <> Eq then res 
+       else 
+          let res = compare tno1 tno2 in
+          if res = 0 then Eq else if res < 0 then Lt else Gt
   | C.MutInd _, _ -> Lt
   | _, C.MutInd _ -> Gt
 
-  | C.MutConstruct (u1, _, _, _), C.MutConstruct (u2, _, _, _) ->
-      compare_uris u1 u2
+  | C.MutConstruct (u1, tno1, cno1, _), C.MutConstruct (u2, tno2, cno2, _) ->
+       let res =  compare_uris u1 u2 in
+       if res <> Eq then res 
+       else 
+          let res = compare (tno1,cno1) (tno2,cno2) in
+          if res = 0 then Eq else if res < 0 then Lt else Gt
   | C.MutConstruct _, _ -> Lt
   | _, C.MutConstruct _ -> Gt
 
@@ -552,7 +561,7 @@ let nonrec_kbo_w (t1, w1) (t2, w2) =
 let nonrec_kbo t1 t2 =
   let w1 = weight_of_term t1 in
   let w2 = weight_of_term t2 in
-  (* 
+  (*
   prerr_endline ("weight1 :"^(string_of_weight w1));
   prerr_endline ("weight2 :"^(string_of_weight w2)); 
   *)
