@@ -33,9 +33,18 @@
 (*                                                                         *)
 (***************************************************************************)
 
-(* $Id: applyTransformation.ml 8829 2008-07-16 18:16:19Z fguidi $ *)
+(* $Id: applyTransformation.ml 9486 2009-01-13 10:54:34Z tassi $ *)
 
-module G = GrafiteAst
+module UM = UriManager
+module C  = Cic
+module Un = CicUniv
+module E  = CicEnvironment
+module TC = CicTypeChecker
+module G  = GrafiteAst
+module GE = GrafiteEngine
+module LS = LibrarySync
+module Ds = CicDischarge
+module PO = ProceduralOptimizer
 
 let mpres_document pres_box =
   Xml.add_xml_declaration (CicNotationPres.print_box pres_box)
@@ -161,7 +170,8 @@ let term2pres ~map_unicode_to_tex n ids_to_inner_sorts annterm =
    remove_closed_substs s
 
 let txt_of_cic_object 
- ~map_unicode_to_tex ?skip_thm_and_qed ?skip_initial_lambdas n style prefix obj 
+ ~map_unicode_to_tex ?skip_thm_and_qed ?skip_initial_lambdas
+ n style ?flavour prefix obj 
 =
   let get_aobj obj = 
      try   
@@ -169,9 +179,13 @@ let txt_of_cic_object
             Cic2acic.acic_object_of_cic_object obj
         in
         aobj, ids_to_inner_sorts, ids_to_inner_types
-     with e -> 
-        let msg = "txt_of_cic_object: " ^ Printexc.to_string e in
-        failwith msg
+     with 
+        | E.Object_not_found uri -> 
+             let msg = "txt_of_cic_object: object not found: " ^ UM.string_of_uri uri in
+             failwith msg
+	| e                     ->
+             let msg = "txt_of_cic_object: " ^ Printexc.to_string e in
+             failwith msg
   in
   match style with
      | G.Declarative      ->
@@ -190,7 +204,7 @@ let txt_of_cic_object
             (CicNotationPres.mpres_of_box bobj)
         )
      | G.Procedural depth ->
-        let obj = ProceduralOptimizer.optimize_obj obj in
+	let obj, info = PO.optimize_obj obj in
         let aobj, ids_to_inner_sorts, ids_to_inner_types = get_aobj obj in
         let term_pp = term2pres ~map_unicode_to_tex (n - 8) ids_to_inner_sorts in
         let lazy_term_pp = term_pp in
@@ -199,12 +213,37 @@ let txt_of_cic_object
          ~map_unicode_to_tex ~term_pp ~lazy_term_pp ~obj_pp in
         let script = 
            Acic2Procedural.procedural_of_acic_object 
-              ~ids_to_inner_sorts ~ids_to_inner_types 
-	      ?depth prefix aobj 
+              ~ids_to_inner_sorts ~ids_to_inner_types ~info
+	      ?depth ?flavour prefix aobj 
   in
         "\n\n" ^ String.concat "" (List.map aux script)
 
-let txt_of_inline_uri ~map_unicode_to_tex style suri prefix =
+let cic_prefix = Str.regexp_string "cic:/"
+let matita_prefix = Str.regexp_string "cic:/matita/"
+let suffixes = [".ind"; "_rec.con"; "_rect.con"; "_ind.con"; ".con"]
+
+let replacements = 
+   let map s = String.length s, s, Str.regexp_string s, "_discharged" ^ s in 
+   List.map map suffixes
+
+let replacement (ok, u) (l, s, x, t) =
+   if ok then ok, u else
+   if Str.last_chars u l = s then true, Str.replace_first x t u else ok, u
+
+let discharge_uri style uri =
+   let template = match style with
+      | G.Declarative  -> "cic:/matita/declarative/"   
+      | G.Procedural _ -> "cic:/matita/procedural/"
+   in
+   let s = UM.string_of_uri uri in
+   if Str.string_match matita_prefix s 0 then uri else
+   let s = Str.replace_first cic_prefix template s in
+   let _, s = List.fold_left replacement (false, s) replacements in 
+   UM.uri_of_string s
+
+let discharge_name s = s ^ "_discharged"
+
+let txt_of_inline_uri ~map_unicode_to_tex style ?flavour prefix suri =
    let print_exc = function
       | ProofEngineHelpers.Bad_pattern s as e ->
            Printexc.to_string e ^ " " ^ Lazy.force s
@@ -212,19 +251,49 @@ let txt_of_inline_uri ~map_unicode_to_tex style suri prefix =
    in
    let dbd = LibraryDb.instance () in   
    let sorted_uris = MetadataDeps.sorted_uris_of_baseuri ~dbd suri in
+   let error uri e =
+      let msg  = 
+         Printf.sprintf 
+            "ERROR IN THE GENERATION OF %s\nEXCEPTION: %s" 
+            (UM.string_of_uri uri) e
+      in
+      Printf.eprintf "%s\n" msg;
+      GrafiteTypes.command_error msg
+   in
    let map uri =
-      try 
-        txt_of_cic_object 
-          ~map_unicode_to_tex 78 style prefix
-          (fst (CicEnvironment.get_obj CicUniv.empty_ugraph uri))
+      Librarian.time_stamp "AT: BEGIN MAP";
+      try
+(* FG: for now the explicit variables must be discharged *)
+        let do_it obj =
+	   let r = txt_of_cic_object ~map_unicode_to_tex 78 style ?flavour prefix obj in
+	   Librarian.time_stamp "AT: END MAP  "; r
+	in
+	let obj, real = 
+	   let s = UM.string_of_uri uri in
+	   if Str.string_match matita_prefix s 0 then begin
+	      Librarian.time_stamp "AT: GETTING OBJECT";
+	      let obj, _ = E.get_obj Un.default_ugraph uri in
+	      Librarian.time_stamp "AT: DONE          ";
+              obj, true
+	   end else
+	      Ds.discharge_uri discharge_name (discharge_uri style) uri
+	in
+	if real then do_it obj else
+	let newuri = discharge_uri style uri in
+	let _lemmas = LS.add_obj ~pack_coercion_obj:CicRefine.pack_coercion_obj newuri obj in
+	do_it obj
       with
-         | e -> 
-            Printf.sprintf "\n(* ERRORE IN STAMPA DI %s\nEXCEPTION: %s *)\n" 
-            (UriManager.string_of_uri uri) (print_exc e)
+         | TC.TypeCheckerFailure s ->
+	    error uri ("failure  : " ^ Lazy.force s)
+         | TC.AssertFailure s      ->
+	    error uri ("assert   : " ^ Lazy.force s)
+         | E.Object_not_found u    ->
+	    error uri ("not found: " ^ UM.string_of_uri u)
+	 | e                       -> error uri (print_exc e)
    in
    String.concat "" (List.map map sorted_uris)
 
-let txt_of_inline_macro ~map_unicode_to_tex style name prefix =
+let txt_of_inline_macro ~map_unicode_to_tex style ?flavour prefix name =
    let suri = 
       if Librarian.is_uri name then name else
       let include_paths = 
@@ -235,12 +304,13 @@ let txt_of_inline_macro ~map_unicode_to_tex style name prefix =
       in
       baseuri ^ "/"
    in
-   txt_of_inline_uri ~map_unicode_to_tex style suri prefix
+   txt_of_inline_uri ~map_unicode_to_tex style ?flavour prefix suri
 
 (****************************************************************************)
 (* procedural_txt_of_cic_term *)
 
 let procedural_txt_of_cic_term ~map_unicode_to_tex n ?depth context term =
+  let term, _info = PO.optimize_term context term in
   let annterm, ids_to_inner_sorts, ids_to_inner_types = 
      try Cic2acic.acic_term_of_cic_term context term
      with e -> 
@@ -257,3 +327,17 @@ let procedural_txt_of_cic_term ~map_unicode_to_tex n ?depth context term =
         ~ids_to_inner_sorts ~ids_to_inner_types ?depth "" context annterm 
   in
   String.concat "" (List.map aux script)
+;;
+
+(****************************************************************************)
+
+let txt_of_macro ~map_unicode_to_tex metasenv context m =
+   GrafiteAstPp.pp_macro
+     ~term_pp:(txt_of_cic_term ~map_unicode_to_tex 80 metasenv context) 
+     ~lazy_term_pp:(fun (f : Cic.lazy_term) ->
+        let t,metasenv,_ = f context metasenv CicUniv.empty_ugraph in
+        txt_of_cic_term ~map_unicode_to_tex 80 metasenv context t)
+     m
+;;
+
+
