@@ -64,19 +64,25 @@ val txt_of_cic_sequent_conclusion:
   map_unicode_to_tex:bool -> output_type:[`Pattern | `Term] -> int ->
    Cic.metasenv -> Cic.conjecture -> string
 
-(* columns, rendering style, name prefix, object *)
+(* columns, rendering style, flavour, name prefix, object *)
 val txt_of_cic_object: 
   map_unicode_to_tex:bool -> 
-  ?skip_thm_and_qed:bool -> 
-  ?skip_initial_lambdas:int -> 
-    int -> GrafiteAst.presentation_style -> string ->
-  Cic.obj ->
+  ?skip_thm_and_qed:bool -> ?skip_initial_lambdas:int -> 
+  int -> GrafiteAst.presentation_style -> ?flavour:Cic.object_flavour ->
+  string -> Cic.obj ->
     string
 
-(* presentation_style, uri or baseuri, name prefix *)
+(* presentation_style, flavour, name prefix, uri or baseuri *)
 val txt_of_inline_macro:
-  map_unicode_to_tex:bool -> 
-   GrafiteAst.presentation_style -> string -> string -> string
+  map_unicode_to_tex:bool -> GrafiteAst.presentation_style -> 
+  ?flavour:Cic.object_flavour -> string -> string ->
+    string
+
+val txt_of_macro:
+  map_unicode_to_tex:bool ->
+    Cic.metasenv ->
+    Cic.context ->
+    (Cic.term, Cic.lazy_term) GrafiteAst.macro -> string
 
 (* columns, rendering depth, context, term *)
 val procedural_txt_of_cic_term: 
