@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: matitaEngine.ml 8721 2008-06-13 12:02:28Z tassi $ *)
+(* $Id: matitaEngine.ml 9287 2008-12-03 22:53:01Z sacerdot $ *)
 
 open Printf
 
@@ -35,7 +35,7 @@ let disambiguate_tactic text prefix_len lexicon_status_ref grafite_status goal t
   GrafiteDisambiguate.disambiguate_tactic
    lexicon_status_ref
    (GrafiteTypes.get_proof_context grafite_status goal)
-   (GrafiteTypes.get_proof_metasenv grafite_status)
+   (GrafiteTypes.get_proof_metasenv grafite_status) (Some goal)
    tac
  in
   GrafiteTypes.set_metasenv metasenv grafite_status,tac
@@ -75,7 +75,8 @@ let eval_ast ?do_heavy_checks lexicon_status
   LexiconSync.alias_diff ~from:lexicon_status new_lexicon_status in
  let _,intermediate_states = 
   List.fold_left
-   (fun (lexicon_status,acc) (k,((v,_) as value)) -> 
+   (fun (lexicon_status,acc) (k,value) -> 
+     let v = LexiconAst.description_of_alias value in
      let b =
       try
        (* this hack really sucks! *)
@@ -141,12 +142,8 @@ let eval_from_stream ~first_statement_only ~include_paths
               (fun (_,alias) ->
                 match alias with
                   None -> ()
-                | Some (k,((v,_) as value)) ->
-                   let newtxt =
-                    DisambiguatePp.pp_environment
-                     (DisambiguateTypes.Environment.add k value
-                       DisambiguateTypes.Environment.empty)
-                   in
+                | Some (k,value) ->
+                   let newtxt = LexiconAstPp.pp_alias value in
                     raise (TryingToAdd newtxt)) new_statuses;
             let grafite_status,lexicon_status =
              match new_statuses with
