@@ -32,8 +32,7 @@ val eval_ast :
    CicNotationPt.term GrafiteAst.reduction, CicNotationPt.term CicNotationPt.obj, string)
    GrafiteAst.statement) ->
   ((GrafiteTypes.status * LexiconEngine.status) *
-   (DisambiguateTypes.domain_item * DisambiguateTypes.codomain_item) option
-  ) list
+   (DisambiguateTypes.domain_item * LexiconAst.alias_spec) option) list
 
 
 (* heavy checks slow down the compilation process but give you some interesting
@@ -57,8 +56,7 @@ val eval_from_stream :
     CicNotationPt.term GrafiteAst.reduction, CicNotationPt.term CicNotationPt.obj, string)
    GrafiteAst.statement -> unit) ->
   ((GrafiteTypes.status * LexiconEngine.status) *
-   (DisambiguateTypes.domain_item * DisambiguateTypes.codomain_item) option
-  ) list
+   (DisambiguateTypes.domain_item * LexiconAst.alias_spec) option) list
 
 (* this callback is called on every grafite command *)
 val set_callback: (GrafiteParser.ast_statement -> unit) -> unit 
