@@ -66,7 +66,7 @@ compile $PWD/helm/software/
 run_tests $PWD/helm/software/matita > LOG 2>LOG.run_tests.err
 
 #insert the db
-cat LOG | grep "\(OK\|FAIL\)" | grep "\(gc-on\|gc-off\)" | grep -v "bad_tests" | grep -v "interactive" |\
+cat LOG | grep "\(OK\|FAIL\)" | grep "\(gc-on\|gc-off\)" | grep -v "bad_tests" | grep "^matitac" | grep -v "interactive" |\
   lua5.1 $PWD/helm/software/matita/scripts/functions.lua log2sql - > INSERT.sql
 cat INSERT.sql | $MYSQL
 
